from turbogears.widgets import Widget
from turbogears.widgets import JSLink

class MochiKit(Widget):
    javascript = [JSLink("openerp", "javascript/MochiKit/MochiKit.js"),
                  JSLink("openerp", "javascript/MochiKit/DragAndDrop.js"),
                  JSLink("openerp", "javascript/MochiKit/Resizable.js"),
                  JSLink("openerp", "javascript/MochiKit/Sortable.js")]

mochikit = MochiKit()




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

from turbojson import jsonify

import turbogears

from turbogears import widgets
from interface import TinyField

class TreeGrid(TinyField):

    template = "openerp.widgets.templates.treegrid"
    params = ['headers', 'showheaders', 'expandall', 'onselection', 'onbuttonclick', 'onheaderclick', 'url', 'url_params']

    css = [widgets.CSSLink("openerp", "css/treegrid.css")]
    javascript = [widgets.JSLink("openerp", "javascript/treegrid.js")]

    def __init__(self, name, model, headers, url, field_parent=None, ids=[], domain=[], context={}, **kw):
        attrs = dict(name=name, model=model, url=url)
        super(TreeGrid, self).__init__(attrs)
        
        self.ids = ids
        self.model = model
        
        self.headers = jsonify.encode(headers)
        
        fields = [field['name'] for field in headers]
        icon_name = headers[0].get('icon')
        
        self.url = url
        self.url_params = dict(model=model, 
                                ids=ids,
                                fields=ustr(fields), 
                                domain=ustr(domain), 
                                context=ustr(context), 
                                field_parent=field_parent,
                                icon_name=icon_name)
        
        self.url_params.update(kw)
        
        def _jsonify(obj):
            
            for k, v in obj.items():
                if isinstance(v, dict):
                    obj[k] = _jsonify(v)
            
            return jsonify.encode(obj)
                
        self.url_params = _jsonify(self.url_params)
        
        self.domain = domain
        self.context = context
        
        self.showheaders = kw.get('showheaders', 1)
        self.onselection = kw.get('onselection')
        self.onbuttonclick = kw.get('onbuttonclick')
        self.onheaderclick = kw.get('onheaderclick')
        self.expandall = kw.get('expandall', 0)
        
# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import os
import re
import copy
import cPickle

import cherrypy

from gettext import translation
from turbogears.i18n import tg_gettext

import rpc

__cache_references = []

def clear():
    for queue, store in __cache_references:
        while queue:
            del store[queue.pop()]

def memoize(limit=100, force=False):

    def memoize_wrapper(func):

        # Don't use cache for development environment
        if not force and cherrypy.config.get('server.environment') == 'development':
            return func

        queue = []
        store = {}
        
        __cache_references.append((queue, store))

        def func_wrapper(*args, **kwargs):
            key = cPickle.dumps((args, kwargs))
            try:
                queue.append(queue.pop(queue.index(key)))
            except ValueError:
                store[key] = func(*args, **kwargs)
                queue.append(key)
                if limit is not None and len(queue) > limit:
                    del store[queue.pop(0)]

            return copy.deepcopy(store[key])

        func_wrapper.func_name = func.func_name
        return func_wrapper

    return memoize_wrapper

@memoize(1000)
def __fields_view_get(model, view_id, view_type, context, hastoolbar, sid):
    return rpc.RPCProxy(model).fields_view_get(view_id, view_type, context, hastoolbar)

def fields_view_get(model, view_id, view_type, context, hastoolbar=False):
    return __fields_view_get(model, view_id, view_type, context, hastoolbar=hastoolbar, sid=cherrypy.session.id)

@memoize(1000)
def __fields_get(model, fields, context, sid):
    return rpc.RPCProxy(model).fields_get(fields, context)

def fields_get(model, fields, context):
    return __fields_get(model, fields, context, sid=cherrypy.session.id)

@memoize(1000)
def __can_write(model, sid):
    proxy = rpc.RPCProxy('ir.model.access')
    try:
        return proxy.check(model, 'write')
    except:
        pass
    return False

def can_write(model):
    return __can_write(model, sid=cherrypy.session.id)

@memoize(10000, True)
def _gettext(key, locale, domain):
    return tg_gettext.tg_gettext(key, locale, domain)

def gettext(key, locale=None, domain=None):
    if key in __TRANSLATABLES:
        return _gettext(key, locale or tg_gettext.get_locale(), 'messages')
    return key

def __load_translatables():

    localedir = tg_gettext.get_locale_dir()

    #XXX: TG bug #1631
    if not localedir:
        from turbogears import config
        config.update({'package': 'openerp'})
        localedir = tg_gettext.get_locale_dir()
    
    result = {}
    for lang in os.listdir(localedir):
        try:
            t = translation(domain='messages', localedir=localedir, languages=[lang])
            result.update(t._catalog)
        except:
            pass

    return result.keys()

__TRANSLATABLES = __load_translatables()

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import re
import time
import datetime as DT

import turbogears as tg

from openerp import rpc

DT_SERVER_FORMATS = {
  'datetime' : '%Y-%m-%d %H:%M:%S',
  'date' : '%Y-%m-%d',
  'time' : '%H:%M:%S'
}

def get_datetime_format(kind="datetime"):
    """Get local datetime format.
    
    @param kind: type (date, time or datetime)
    @return: string
    
    @todo: cache formats to improve performance.
    @todo: extend user preferences to allow customisable date format (tiny server).
    """

    fmt = "%H:%M:%S"
    
    if kind != "time":
        fmt = tg.i18n.format.get_date_format("short", rpc.session.locale).replace("%y", "%Y")
    
    if kind == "datetime":
        fmt += " %H:%M:%S"
    
    return fmt

def format_datetime(value, kind="datetime", as_timetuple=False):
    """Convert date value to the local datetime considering timezone info.

    @param value: the date value
    @param kind: type of the date value (date, time or datetime)
    @param as_timetuple: return timetuple
    
    @type value: basestring or time.time_tuple)
    
    @return: string or timetuple
    """

    server_format = DT_SERVER_FORMATS[kind]
    local_format = get_datetime_format(kind)
    
    if not value:
        return ''
    
    if isinstance(value, (time.struct_time, tuple)):
        value = time.strftime(server_format, value)

    value = value.strip()

    # remove trailing miliseconds
    value = re.sub("(.*?)(\s+\d{2}:\d{2}:\d{2})(\.\d+)?$", "\g<1>\g<2>", value)

    # add time part in value if missing
    if kind == 'datetime' and not re.search('\s+\d{2}:\d{2}:\d{2}?$', value):
        value += ' 00:00:00'

    # remove time part from value
    elif kind == 'date':
        value = re.sub('\s+\d{2}:\d{2}:\d{2}(\.\d+)?$', '', value)

    value = time.strptime(value, server_format)

    if kind == "datetime" and 'tz' in rpc.session.context:
        try:
            import pytz
            lzone = pytz.timezone(str(rpc.session.context['tz']))
            szone = pytz.timezone(str(rpc.session.timezone))
            dt = DT.datetime(value[0], value[1], value[2], value[3], value[4], value[5], value[6])
            sdt = szone.localize(dt, is_dst=True)
            ldt = sdt.astimezone(lzone)
            value = ldt.timetuple()
        except:
            pass

    if as_timetuple:
        return value
    
    return time.strftime(local_format, value)

def parse_datetime(value, kind="datetime", as_timetuple=False):
    """Convert date value to the server datetime considering timezone info.

    @param value: the date value
    @param kind: type of the date value (date, time or datetime)
    @param as_timetuple: return timetuple
    
    @type value: basestring or time.time_tuple)
    
    @return: string or timetuple
    """
    
    server_format = DT_SERVER_FORMATS[kind]
    local_format = get_datetime_format(kind)

    if not value:
        return False

    if isinstance(value, (time.struct_time, tuple)):
        value = time.strftime(local_format, value)

    try:
        value = time.strptime(value, local_format)
    except:
        try:
            dt = list(time.localtime())
            dt[2] = int(value)
            value = tuple(dt)
        except:
            return False

    if kind == "datetime" and 'tz' in rpc.session.context:
        try:
            import pytz
            lzone = pytz.timezone(rpc.session.context['tz'])
            szone = pytz.timezone(rpc.session.timezone)
            dt = DT.datetime(value[0], value[1], value[2], value[3], value[4], value[5], value[6])
            ldt = lzone.localize(dt, is_dst=True)
            sdt = ldt.astimezone(szone)
            value = sdt.timetuple()
        except:
            pass

    if as_timetuple:
        return value
    
    return time.strftime(server_format, value)

def format_decimal(value, digits=2):
    return tg.i18n.format_decimal(value or 0.0, digits)

def parse_decimal(value):
    
    if isinstance(value, basestring):
        
        value = ustr(value)
        
        #deal with ' ' instead of u'\xa0' (SP instead of NBSP as grouping char)
        value = value.replace(' ', '')
        try:
            value = tg.i18n.format.parse_decimal(value)
        except ValueError, e:
            pass

    if not isinstance(value, float):
        return float(value)
    
    return value

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import re
import time
import base64
import xmlrpclib

import cherrypy
import pkg_resources

from turbogears import expose
from turbogears import widgets
from turbogears import controllers
from turbogears import validators, validate, error_handler
from turbogears import redirect

from openerp import rpc
from openerp import common


def get_lang_list():
    langs = [('en_EN', 'English')]
    try:
        return langs + (rpc.session.execute_db('list_lang') or [])
    except Exception, e:
        pass
    return langs

def get_db_list():
    try:
        return rpc.session.execute_db('list') or []
    except:
        return []

class FormCreate(widgets.TableForm):
    name = "create"
    string = _('Create new database')
    action = '/database/do_create'
    submit_text = _('OK')
    form_attrs = {'onsubmit': 'return on_create()'}
    fields = [widgets.PasswordField(name='password', label=_('Super admin password:'), validator=validators.NotEmpty()),
              widgets.TextField(name='dbname', label=_('New database name:'), validator=validators.NotEmpty()),
              widgets.CheckBox(name='demo_data', label=_('Load Demonstration data:'), default=True, validator=validators.Bool(if_empty=False)),
              widgets.SingleSelectField(name='language', options=get_lang_list, validator=validators.String(), label=_('Default Language:')),
              widgets.PasswordField(name='admin_password', label=_('Administrator password:'), validator=validators.NotEmpty()),
              widgets.PasswordField(name='confirm_password', label=_('Confirm password:'), validator=validators.NotEmpty())]

    validator = validators.Schema(chained_validators=[validators.FieldsMatch("admin_password","confirm_password")])

class FormDrop(widgets.TableForm):
    name = "drop"
    string = _('Drop database')
    action = '/database/do_drop'
    submit_text = _('OK')
    form_attrs = {'onsubmit': 'return window.confirm("%s")' % _("Do you really want to drop the selected database?")}
    fields = [widgets.SingleSelectField(name='dbname', options=get_db_list, label=_('Database:'), validator=validators.String(not_empty=True)),
              widgets.PasswordField(name='password', label=_('Password:'), validator=validators.NotEmpty())]

class FormBackup(widgets.TableForm):
    name = "backup"
    string = _('Backup database')
    action = '/database/do_backup'
    submit_text = _('OK')
    fields = [widgets.SingleSelectField(name='dbname', options=get_db_list, label=_('Database:'), validator=validators.String(not_empty=True)),
              widgets.PasswordField(name='password', label=_('Password:'), validator=validators.NotEmpty())]

class FormRestore(widgets.TableForm):
    name = "restore"
    string = _('Restore database')
    action = '/database/do_restore'
    submit_text = _('OK')
    fields = [widgets.FileField(name="filename", label=_('File:')),
              widgets.PasswordField(name='password', label=_('Password:'), validator=validators.NotEmpty()),
              widgets.TextField(name='dbname', label=_('New database name:'), validator=validators.NotEmpty())]

class FormPassword(widgets.TableForm):
    name = "password"
    string = _('Change Administrator Password')
    action = '/database/do_password'
    submit_text = _('OK')
    fields = [widgets.PasswordField(name='old_password', label=_('Old Password:'), validator=validators.NotEmpty()),
              widgets.PasswordField(name='new_password', label=_('New Password:'), validator=validators.NotEmpty()),
              widgets.PasswordField(name='confirm_password', label=_('Confirm Password:'), validator=validators.NotEmpty())]

    validator = validators.Schema(chained_validators=[validators.FieldsMatch("new_password","confirm_password")])


_FORMS = {
    'create': FormCreate(),
    'drop': FormDrop(),
    'backup': FormBackup(),
    'restore': FormRestore(),
    'password': FormPassword()
}

class Database(controllers.Controller):

    @expose()
    def index(self, *args, **kw):
        raise redirect('/database/create')

    @expose(template="openerp.subcontrollers.templates.database")
    def create(self, tg_errors=None, **kw):
        form = _FORMS['create']
        return dict(form=form)

    @expose()
    @validate(form=_FORMS['create'])
    @error_handler(create)    
    def do_create(self, password, dbname, admin_password, confirm_password, demo_data=False, language=None, **kw):

        if not re.match('^[a-zA-Z][a-zA-Z0-9_]+$', dbname):
            raise common.warning(_('The database name must contain only normal characters or "_".\nYou must avoid all accents, space or special characters.'), _('Bad database name!'))

        ok = False
        try:
            dblist = rpc.session.listdb()
            if dbname in dblist:
                raise Exception('DbExist')

            res = rpc.session.execute_db('create', password, dbname, demo_data, language, admin_password)
            while True:
                try:
                    progress, users = rpc.session.execute_db('get_progress', password, res)
                    if progress == 1.0:
                        for x in users:
                            if x['login'] == 'admin':
                                rpc.session.login(dbname, 'admin', password)
                                ok = True
                        break
                    else:
                        time.sleep(1)
                except:
                    raise Exception('DbFailed')
        except Exception, e:
            if e.args == ('DbExist',):
                raise common.warning(_("Could not create database."), _('Database already exists!'))
            elif e.args == ('DbFailed'):
                raise common.warning(_("The server crashed during installation.\nWe suggest you to drop this database."), 
                                     _("Error during database creation!"))
            elif getattr(e, 'faultCode', False) == 'AccessDenied':
                raise common.warning(_('Bad database administrator password!'), _("Could not create database."))
            else:
                raise common.warning(_("Could not create database."))

        if ok:
            raise redirect('/')
        raise redirect('/login', db=dbname)

    @expose(template="openerp.subcontrollers.templates.database")
    def drop(self, tg_errors=None, **kw):
        form = _FORMS['drop']
        return dict(form=form)

    @expose()
    @validate(form=_FORMS['drop'])
    @error_handler(drop)    
    def do_drop(self, dbname, password, **kw):
        try:
            rpc.session.execute_db('drop', password, dbname)
        except Exception, e:
            if getattr(e, 'faultCode', False) == 'AccessDenied':
                raise common.warning(_('Bad database administrator password!'), _("Could not drop database."))
            else:
                raise common.warning(_("Couldn't drop database"))

        raise redirect("/database/drop")

    @expose(template="openerp.subcontrollers.templates.database")
    def backup(self, tg_errors=None, **kw):
        form = _FORMS['backup']
        return dict(form=form)

    @expose()
    @validate(form=_FORMS['backup'])
    @error_handler(backup)    
    def do_backup(self, dbname, password, **kw):
        try:
            res = rpc.session.execute_db('dump', password, dbname)
            if res:
                cherrypy.response.headers['Content-Type'] = "application/data"
                cherrypy.response.headers['Content-Disposition'] = 'filename="' + dbname + '.dump"';
                return base64.decodestring(res)
        except Exception, e:
            raise common.warning(_("Could not create backup."))

        raise redirect('/login')

    @expose(template="openerp.subcontrollers.templates.database")
    def restore(self, tg_errors=None, **kw):
        form = _FORMS['restore']
        return dict(form=form)

    @expose()
    @validate(form=_FORMS['restore'])
    @error_handler(restore)    
    def do_restore(self, filename, password, dbname, **kw):
        try:
            data = base64.encodestring(filename.file.read())
            res = rpc.session.execute_db('restore', password, dbname, data)
        except Exception, e:
            if getattr(e, 'faultCode', False) == 'AccessDenied':
                raise common.warning(_('Bad database administrator password!'), _("Could not restore database."))
            else:
                raise common.warning(_("Couldn't restore database"))

        raise redirect('/login', db=dbname)

    @expose(template="openerp.subcontrollers.templates.database")
    def password(self, tg_errors=None, **kw):
        form = _FORMS['password']
        return dict(form=form)

    @validate(form=_FORMS['password'])
    @error_handler(password)
    @expose()
    def do_password(self, old_password, new_password, confirm_password, **kw):
        try:
            res = rpc.session.execute_db('change_admin_password', old_password, new_password)
        except Exception,e:
            if getattr(e, 'faultCode', False) == 'AccessDenied':
                raise common.warning(_("Could not change super admin password."), _('Bad password provided!'))
            else:
                raise common.warning(_("Error, password not changed."))

        raise redirect('/login')

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

"""
This module implementes search view for a tiny model. Currently it simply displays
list view of the given model.
"""
import cherrypy

from turbogears import expose
from turbogears import widgets
from turbogears import redirect
from turbogears import controllers
from turbogears import validators
from turbogears import validate

from openerp import rpc
from openerp import tools
from openerp import common

from openerp import widgets as tw
from openerp import widgets_search as tws

from openerp.tinyres import TinyResource

from openerp.utils import TinyDict
from openerp.utils import TinyForm
from openerp.utils import TinyFormError

from form import Form

class Search(Form):

    @expose(template="openerp.subcontrollers.templates.search")
    def create(self, params, tg_errors=None):

        params.view_mode = ['tree', 'form']
        params.view_type = 'tree'

        params.offset = params.offset or 0
        params.limit = params.limit or 20
        params.count = params.count or 0

        params.editable = 0

        form = self.create_form(params, tg_errors)

        # don't show links in list view, except the do_select link
        form.screen.widget.show_links = 0

        return dict(form=form, params=params, show_header_footer=False)

    @expose()
    def new(self, model, source=None, kind=0, text=None, domain=[], context={}):
        """Create new search view...

        @param model: the model
        @param source: the source, in case of m2m, m2o search
        @param kind: 0=normal, 1=m2o, 2=m2m
        @param text: do `name_search` if text is provided
        @param domain: the domain
        @param context: the context
        """

        params = TinyDict()

        params.model = model
        params.domain = domain
        params.context = context

        params.source = source
        params.selectable = kind

        ctx = rpc.session.context.copy()
        ctx.update(params.context or {})
        params.ids = []
        proxy = rpc.RPCProxy(model)
        ids = proxy.name_search(text or '', params.domain or [], 'ilike', ctx)
        if ids:
            params.ids = [id[0] for id in ids]
            params.count = len(ids)

        return self.create(params)

    @expose('json')
    def eval_domain_and_context(self, **kw):
        params, data = TinyDict.split(kw)
        
        domain = params.domain
        context = params.context

        parent_context = params.parent_context or {}
        parent_context.update(rpc.session.context.copy())

        try:
            ctx = TinyForm(**kw).to_python()
            pctx = ctx
        except TinyFormError, e:
            return dict(error_field=e.field, error=ustr(e))
        except Exception, e:
            return dict(error=ustr(e))

        prefix = params.prefix
        if prefix:
            ctx = ctx.chain_get(prefix)

        if prefix and '/' in prefix:
            prefix = prefix.rsplit('/', 1)[0]
            pctx = pctx.chain_get(prefix)

        ctx['parent'] = pctx
        ctx['context'] = parent_context

        if isinstance(domain, basestring):
            domain = eval(domain, ctx)  
        
        if isinstance(context, basestring):
            if not context.startswith('{'):
                context = "dict(%s)"%context
                ctx['dict'] = dict # required
            
            ctx['active_id'] = params.parent_id or False
            context = eval(context, ctx)

#           Fixed many2one pop up in listgrid when value is None.
            for key, val in context.items():
                if val==None:
                    context[key] = False

        return dict(domain=ustr(domain), context=ustr(context))

    @expose('json')
    def ok(self, **kw):
        params, data = TinyDict.split(kw)

        ids = [int(id) for id in data.get('search_list', [])]
        return dict(ids=ids)

    @expose('json')
    def get_name(self, model, id):
        name = tw.many2one.get_name(model, id)
        if not name:
            name=''
        return dict(name=name)

    @expose('json')
    def get_matched(self, model, text, **kw):
        params, data = TinyDict.split(kw)

        domain = params.domain or []
        context = params.context or {}
        
        ctx = rpc.session.context.copy()
        ctx.update(context)

        proxy = rpc.RPCProxy(model)
        values = proxy.name_search(text, domain, 'ilike', ctx)
        
        return dict(values=values)


# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import time

from turbogears.widgets import CompoundWidget

from openerp import tools
from openerp.utils import TinyDict
from openerp.widgets.interface import TinyField

from form import Form

class Record(dict):
    
    def __init__(self, params):
        
        self.params = TinyDict(**params.copy())
        self.params.view_type = 'form'
        
        form = Form().create_form(self.params)
        
        record = self._make_record(form)
        self.clear()
        
        self.update(record.copy())
        self['id'] = params.id or False
             
    def _make_record(self, parent=None):
        parent = parent or self

        for wid in parent.iter_member_widgets():

            if isinstance(wid, TinyField) and wid.name and not wid.name.endswith('/'):
                self[wid.name] = wid.get_value()

            elif isinstance(wid, CompoundWidget):
                self._make_record(wid)
    
        params, data = TinyDict.split(self)
        return data
    
    def expr_eval(self, expr, source=None):
        
        if not isinstance(expr, basestring):
            return expr
        
        d = {}
        for name, value in self.items():
            d[name] = value

        d['current_date'] = time.strftime('%Y-%m-%d')
        d['time'] = time
        d['context'] = self.params.context or {}
        d['active_id'] = self.get('id', False)

        val = tools.expr_eval(expr, d)
        return val

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import cherrypy
import turbogears as tg

from openerp.widgets_search.search import Search

from screen import Screen
from sidebar import Sidebar

class ViewForm(tg.widgets.Form):

    template = "openerp.widgets.templates.viewform"

    params = ['limit', 'offset', 'count', 'search_domain', 'search_data']
    member_widgets = ['screen', 'search', 'sidebar']    
    javascript = [tg.widgets.JSLink("openerp", "javascript/form.js", location=tg.widgets.js_location.bodytop),
                  tg.widgets.JSLink("openerp", "javascript/form_state.js", location=tg.widgets.js_location.bodytop),
                  tg.widgets.JSLink("openerp", "javascript/m2o.js", location=tg.widgets.js_location.bodytop),
                  tg.widgets.JSLink("openerp", "javascript/m2m.js", location=tg.widgets.js_location.bodytop),
                  tg.widgets.JSLink("openerp", "javascript/o2m.js", location=tg.widgets.js_location.bodytop),
                  tg.widgets.JSLink("openerp", "javascript/textarea.js", location=tg.widgets.js_location.bodytop),
                  tg.widgets.JSLink("openerp", "javascript/binary.js", location=tg.widgets.js_location.bodytop),
                  tg.widgets.JSLink("openerp", "javascript/charts.js", location=tg.widgets.js_location.bodytop),
                  tg.widgets.JSLink("openerp", "javascript/swfobject.js", location=tg.widgets.js_location.bodytop)]

    def __init__(self, params, **kw):
        super(ViewForm, self).__init__(**kw)
        
        # save reference of params dictionary in requeste
        cherrypy.request.terp_params = params
        cherrypy.request.terp_fields = []

        editable = params.editable
        readonly = params.readonly
        
        if editable is None:
            editable = True
            
        if readonly is None:
            readonly = False
            
        self.screen = Screen(prefix='', hastoolbar=True, editable=editable, readonly=readonly, 
                             selectable=params.selectable or 2)

        self.sidebar = Sidebar(self.screen.model, self.screen.toolbar, self.screen.id, 
                               self.screen.view_type, self.screen.view_type != 'form', 
                               self.screen.context)

        self.is_dashboard = getattr(cherrypy.request, '_terp_dashboard', False)
        
        self.search = None
        
        if params.view_type in ('tree', 'graph'):
            self.search = Search(model=params.model, domain=params.domain, 
                                 context=params.context, values=params.search_data or {})
            
        if params.view_type == 'tree':
            self.screen.id = False
            
        if params.context and '_view_name' in params.context:
            self.screen.string = params.context.get('_view_name')

        # get the actual pager data
        self.limit = self.screen.limit
        self.offset = self.screen.offset        
        self.count = self.screen.count
        
        self.search_domain = params.search_domain
        self.search_data = params.search_data
        
        self.fields = cherrypy.request.terp_fields
        
# vim: ts=4 sts=4 sw=4 si et





from turbogears import startup
from turbogears.i18n import tg_gettext

from turbogears.widgets import Widget
from turbogears.widgets import JSLink

def _get_locale():
    lang = tg_gettext.get_locale()
    if len(lang) == 2:
        lang = lang + '_' + lang
    if len(lang) > 2:
        country = lang[3:].upper()
        lang = lang[:2] + "_" + country
    return lang

class JSCatelog(JSLink):
    def update_params(self, d):
        super(JSCatelog, self).update_params(d)
        lang = _get_locale()
        if tg_gettext.is_locale_supported(lang):
            d["link"] = "/%sstatic/javascript/i18n/%s.js" % (startup.webpath, lang)

class JSI18n(Widget):
    javascript = [JSLink('openerp', 'javascript/i18n/i18n.js'),
                  JSCatelog('openerp', 'javascript/i18n/en_EN.js'),]

js_i18n = JSI18n()


# Auto generate language files from gettext catalogs.

import os
import pkg_resources
from turbojson import jsonify

def __generate_catalog(locale):

    if not tg_gettext.is_locale_supported(locale):
        return

    fname = pkg_resources.resource_filename("openerp",  "static/javascript/i18n/%s.js" % locale)
    cname = os.path.join(tg_gettext.get_locale_dir(), locale, 'LC_MESSAGES', 'messages.mo')

    if os.path.exists(fname) and os.path.getmtime(fname) >= os.path.getmtime(cname):
        return

    print "Generating JavaScript i18n message catalog for %s..." % locale
    messages = {}
    try:
        messages = tg_gettext.get_catalog(locale=locale)._catalog
        messages.pop("")
    except Exception, e:
        pass
    messages = jsonify.encode(messages)

    catalog = """
// Auto generated file. Please don't modify.
var MESSAGES = %(messages)s;

""" % dict(messages=messages)

    try:
        fo = open(fname, 'w')
        fo.write(catalog)
        fo.close()
    except Exception, e:
        pass

def __generate_catalogs():
    for lang in os.listdir(tg_gettext.get_locale_dir()):
        __generate_catalog(lang)

__generate_catalogs()

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

from turbogears import controllers
from turbogears import expose
from turbogears import redirect
#from turbogears import widgets
#from turbogears import validators
from turbogears import validate
from turbogears import flash

from turbogears import widgets as tg_widgets
from turbogears import validators as tg_validators

import pkg_resources
import cherrypy

from openerp import rpc
#from openerp import cache
from openerp import common
from openerp.utils import TinyDict
from openerp.tinyres import TinyResource
from openerp import widgets as tw

from form import Form


#rpc.session = rpc.RPCSession('localhost', '8070', 'socket', storage=cherrypy.session)

class State(Form):
    
    path = '/workflow/state'    # mapping from root
    
    @expose(template="openerp.subcontrollers.templates.wkf_popup")
    def create(self, params, tg_errors=None):  
              
        params.path = self.path
        params.function = 'create_state'
        
        if params.id and cherrypy.request.path == self.path + '/view':
            params.load_counter = 2
        
        form = self.create_form(params, tg_errors)  
        
        field = form.screen.widget.get_widgets_by_name('wkf_id')[0]     
        field.set_value(params.wkf_id or False)
        field.readonly = True
         
        form.hidden_fields = [tg_widgets.HiddenField(name='wkf_id', default=params.wkf_id)]
        vals = getattr(cherrypy.request, 'terp_validators', {})
        vals['wkf_id'] = tw.validators.Int()
        
        hide = []
        
        hide += form.screen.widget.get_widgets_by_name('out_transitions')
        hide += form.screen.widget.get_widgets_by_name('in_transitions')
        hide += form.screen.widget.get_widgets_by_name('', kind=tw.form.Separator)
        
        for w in hide:
            w.visible = False

        return dict(form=form, params=params, show_header_footer=False)

    @expose()    
    def edit(self, **kw):
        
        params, data = TinyDict.split(kw)
        
        if not params.model:
            params.update(kw)
       
        params.view_mode = ['form']
        params.view_type = 'form'        
        params.editable = True
        
        return self.create(params)
    
    @expose()
    def delete(self, id, **kw):
        
        error_msg = None         
        proxy = rpc.RPCProxy('workflow.activity')                    
        res_act = proxy.unlink(int(id))                

        if not res_act:
            error_msg = _('Could not delete state')
                    
        return dict(error = error_msg)
    
    @expose('json')
    def get_info(self, id, **kw):
        
        proxy_act = rpc.RPCProxy('workflow.activity')
        search_act = proxy_act.search([('id', '=', int(id))], 0, 0, 0, rpc.session.context)
        data = proxy_act.read(search_act, [], rpc.session.context)
    
        return dict(data=data[0])

class Connector(Form):
    
    path = '/workflow/connector'    # mapping from root
    
    @expose(template="openerp.subcontrollers.templates.wkf_popup")
    def create(self, params, tg_errors=None):
        
        params.path = self.path
        params.function = 'update_connection'
        
        if params.id and cherrypy.request.path == self.path + '/view':
            params.load_counter = 2
            
        form = self.create_form(params, tg_errors)
        
        field_act_from = form.screen.widget.get_widgets_by_name('act_from')[0]
        field_act_from.set_value(params.start or False)
        field_act_from.readonly = True
        
        field_act_to = form.screen.widget.get_widgets_by_name('act_to')[0]
        field_act_to.set_value(params.end or False)
        field_act_to.readonly = True
        
        form.hidden_fields = [tg_widgets.HiddenField(name='act_from', default=params.start),
                            tg_widgets.HiddenField(name='act_to', default=params.end)]
        
        vals = getattr(cherrypy.request, 'terp_validators', {})
        vals['act_from'] = tw.validators.Int()
        vals['act_to'] = tw.validators.Int()
        
        return dict(form=form, params=params, show_header_footer=False)
                
    @expose()    
    def edit(self, **kw):
        
        params, data = TinyDict.split(kw)
        
        if not params.model:
            params.update(kw)
            
        params.view_mode = ['form']
        params.view_type = 'form'
        params.editable = True
        
        return self.create(params)
    
    @expose('json')
    def delete(self, id, **kw):
        
        error_msg = None
        proxy = rpc.RPCProxy('workflow.transition')
        res_tr = proxy.unlink(int(id))
            
        if not res_tr:    
            error_msg = _('Could not delete state')
        
        return dict(error=error_msg)
    
    @expose('json')
    def auto_create(self, act_from, act_to, **kw):
        
        proxy_tr = rpc.RPCProxy('workflow.transition')
        id = proxy_tr.create({'act_from': act_from, 'act_to': act_to})
        data = proxy_tr.read(id, [], rpc.session.context);
        
        if id>0:
            return dict(flag=True,data=data)
        else:
            return dict(flag=False)
    
    @expose('json')
    def get_info(self, id, **kw):
                
        proxy_tr = rpc.RPCProxy('workflow.transition')
        search_tr = proxy_tr.search([('id', '=', int(id))], 0, 0, 0, rpc.session.context)
        data = proxy_tr.read(search_tr, [], rpc.session.context)
        
        return dict(data=data[0])
    
    @expose('json')
    def change_ends(self, id, field, value):     
           
        proxy_tr = rpc.RPCProxy('workflow.transition')
        id = proxy_tr.write([int(id)], {field: int(value)}, rpc.session.context)
        return dict()


class Workflow(Form):
    
    path = '/workflow'
    
    @expose(template="openerp.subcontrollers.templates.workflow")
    def index(self, model, id=None):
        
        proxy = rpc.RPCProxy("workflow")
        if id:
            ids = proxy.search([('id', '=', id)], 0, 0, 0, rpc.session.context)
        else:
            ids = proxy.search([('osv', '=', model)], 0, 0, 0, rpc.session.context)
        
        if not ids:
            raise common.message(_('No workflow associated!'))
        
        wkf = proxy.read(ids, [], rpc.session.context)[0]
        return dict(wkf=wkf, show_header_footer=False)
    
    @expose('json')
    def get_info(self, id, **kw):
        
        proxy = rpc.RPCProxy("workflow")
        search_ids = proxy.search([('id', '=' , int(id))], 0, 0, 0, rpc.session.context) 
        graph_search = proxy.graph_get(search_ids[0], (140, 180), rpc.session.context) 
         
        nodes = graph_search['nodes']
        transitions = graph_search['transitions']
        
        connectors = {}
        list_tr = [];
            
        for tr in transitions:
            list_tr.append(tr)
            t = connectors.setdefault(tr,{})
            t['id'] = tr
            t['s_id'] = transitions[tr][0]
            t['d_id'] = transitions[tr][1]
            
        proxy_tr = rpc.RPCProxy("workflow.transition")
        search_trs = proxy_tr.search([('id', 'in', list_tr)], 0, 0, 0, rpc.session.context)
        data_trs = proxy_tr.read(search_trs, ['signal', 'condition', 'act_from', 'act_to'], rpc.session.context)

        for tr in data_trs:
            t = connectors.get(tr['id'])
            t['signal'] = tr['signal']
            t['condition'] = tr['condition']
            t['source'] = tr['act_from'][1]
            t['destination'] = tr['act_to'][1]
        
        proxy_act = rpc.RPCProxy("workflow.activity")
        search_acts = proxy_act.search([('wkf_id', '=', int(id))], 0, 0, 0, rpc.session.context) 
        data_acts = proxy_act.read(search_acts, ['action', 'kind', 'flow_start', 'flow_stop', 'subflow_id'], rpc.session.context)
        
        for act in data_acts:
            n = nodes.get(str(act['id'])) 
            n['id'] = act['id']
            n['flow_start'] = act['flow_start']
            n['flow_stop'] = act['flow_stop']
            n['action'] = act['action']
            n['kind'] = act['kind']
            n['subflow_id'] = act['subflow_id']
            
        return dict(nodes=nodes,conn=connectors)
    
    state = State()
    connector = Connector()
    

class WorkflowList(controllers.Controller, TinyResource):

    @expose(template="openerp.subcontrollers.templates.wkf_list")
    def index(self, model, active=False):
        
        params = TinyDict()
        params.model = 'workflow'
        params.view_mode = ['tree']
        
        params.domain = [('osv', '=', model)]
        
        screen = tw.screen.Screen(params, selectable=1)
        screen.widget.pageable = False
        
        return dict(screen=screen, model=model, active=active, show_header_footer=False)
    
    @expose()
    def create(self, model, **kw):
        
        wkf_name = kw.get('name')
        on_create = kw.get('on_create')
        
        if not wkf_name:
            raise redirect('/workflowlist', model=model)
        
        proxy = rpc.RPCProxy('workflow')
        proxy.create(dict(osv=model, name=wkf_name, on_create=on_create))
        
        raise redirect('/workflowlist', model=model)
    
    @expose()
    def delete(self, model, id):
        
        id = int(id)
        
        proxy = rpc.RPCProxy('workflow')
        proxy.unlink(id)
        
        raise redirect('/workflowlist', model=model)
    
    @expose()
    def activate(self, model, id):
        
        activate_id = int(id)
        
        proxy = rpc.RPCProxy('workflow')
        search_ids = proxy.search([('osv', '=', model)], 0, 0, 0, rpc.session.context)
        
        for id in search_ids:
            if id==activate_id:                
                proxy.write([id], {'on_create': True})
            else:
                proxy.write([id], {'on_create': False})
                
        raise redirect('/workflowlist', model=model)

# vim: ts=4 sts=4 sw=4 si et

        



###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import re

from turbogears import expose
from turbogears import redirect
from turbogears import widgets
from turbogears import controllers
from turbogears import validators
from turbogears import validate

import cherrypy

from openerp import rpc
from openerp import tools
from openerp import common

from openerp import widgets as tw
from openerp.tinyres import TinyResource
from openerp.utils import TinyDict

import form

class Wizard(controllers.Controller, TinyResource):

    def execute(self, params):
        
        action = params.name
        model = params.model
        state = params.state
        datas = params.datas

        form = None
        buttons = []

        if model:
            action = model.replace('wizard.', '', 1)
        else:
            model = 'wizard.' + action

        params.name = action
        params.model = model
        params.view_mode = []

        if 'form' not in datas:
            datas['form'] = {}

        wiz_id = params.wiz_id or rpc.session.execute('wizard', 'create', action)

        while state != 'end':

            ctx = rpc.session.context.copy()
            ctx.update(params.context or {})

            res = rpc.session.execute('wizard', 'execute', wiz_id, datas, state, ctx)

            if 'datas' in res:
                datas['form'].update(res['datas'])
            else:
                res['datas'] = {}

            if res['type']=='form':
                form = tw.form_view.ViewForm(params, name="view_form", action="/wizard/action")
                
                fields = res['fields']
                form_values = {}
                
                for f in fields:
                    if 'value' in fields[f]:
                        form_values[f] = fields[f]['value']

                form_values.update(datas['form'])
                
                datas['form'] = form_values

                res['datas'].update(datas['form'])
                form.screen.add_view(res)

                # store datas in _terp_datas
                form.hidden_fields = [widgets.HiddenField(name='_terp_datas', default=ustr(datas)),
                                      widgets.HiddenField(name='_terp_state2', default=state),
                                      widgets.HiddenField(name='_terp_wiz_id', default=wiz_id)]

                buttons = res.get('state', [])
                buttons = [(b[0], re.sub('_(?!_)', '', b[1])) for b in buttons] # remove mnemonic
                params.state = state
                
                return dict(form=form, buttons=buttons)

            elif res['type']=='action':
                from openerp.subcontrollers import actions

                act_res = actions.execute(res['action'], **datas)
                if act_res:
                    return act_res

                state = res['state']

            elif res['type']=='print':
                from openerp.subcontrollers import actions
                
                datas['report_id'] = res.get('report_id', False)
                if res.get('get_id_from_action', False):
                    backup_ids = datas['ids']
                    datas['ids'] = datas['form']['ids']

                return actions.execute_report(res['report'], **datas)

            elif res['type']=='state':
                state = res['state']

        raise redirect('/wizard/end')

    @expose(template="openerp.subcontrollers.templates.wizard")
    def create(self, params, tg_errors=None):

        if tg_errors:
            form = cherrypy.request.terp_form
            buttons = cherrypy.request.terp_buttons
            return dict(form=form, buttons=buttons)

        return self.execute(params)

    @expose()
    def end(self, **kw):

        if 'wizard_parent_params' in cherrypy.session:
            frm = cherrypy.session['wizard_parent_form']
            params = cherrypy.session['wizard_parent_params']

            frm = eval('cherrypy.root' + frm.replace('/', '.'))
            return frm.create(params)

        return """<html>
            <head>
                <script type="text/javascript">
                    if (window.opener) {
                        window.opener.setTimeout("window.location.reload()", 0);
                        window.close();
                    } else {
                        window.location.href = "/";
                    }
                </script>
            </head>
            <body>
            </body>
        </html>
        """

    def get_validation_schema(self):

        kw = cherrypy.request.params
        params, datas = TinyDict.split(kw)

        params.state = params.state2

        cherrypy.request.terp_validators = {}

        res = self.execute(params)
        
        form = res['form']
        buttons = res.get('buttons', [])

        cherrypy.request.terp_form = form
        cherrypy.request.terp_buttons = buttons

        vals = cherrypy.request.terp_validators
        keys = vals.keys()
        for k in keys:
            if k not in kw:
                vals.pop(k)

        form.validator = validators.Schema(**vals)
        return form

    @expose()
    @validate(form=get_validation_schema)
    def report(self, tg_errors=None, tg_exceptions=None, **kw):
        
        if tg_exceptions:
            raise tg_exceptions

        params, datas = TinyDict.split(kw)
        params.datas['form'].update(datas)
        
        if tg_errors:
            return self.create(params, tg_errors=tg_errors)

        return self.execute(params)

    @expose()
    @validate(form=get_validation_schema)
    def action(self, tg_errors=None, tg_exceptions=None, **kw):

        if tg_exceptions:
            raise tg_exceptions

        params, datas = TinyDict.split(kw)
        params.datas['form'].update(datas)
        
        return self.create(params, tg_errors=tg_errors)
    
# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import math
import time
import datetime
import xml.dom.minidom

import cherrypy
import turbogears as tg

from openerp import rpc
from openerp import tools
from openerp import format
from openerp.utils import TinyDict

from openerp.widgets import interface

from utils import Day
from utils import parse_datetime

COLOR_PALETTE = ['#f57900', '#cc0000', '#d400a8', '#75507b', '#3465a4', '#73d216', '#c17d11', '#edd400',
                 '#fcaf3e', '#ef2929', '#ff00c9', '#ad7fa8', '#729fcf', '#8ae234', '#e9b96e', '#fce94f',
                 '#ff8e00', '#ff0000', '#b0008c', '#9000ff', '#0078ff', '#00ff00', '#e6ff00', '#ffff00',
                 '#905000', '#9b0000', '#840067', '#510090', '#0000c9', '#009b00', '#9abe00', '#ffc900',]

_colorline = ['#%02x%02x%02x' % (25+((r+10)%11)*23,5+((g+1)%11)*20,25+((b+4)%11)*23) for r in range(11) for g in range(11) for b in range(11) ]
def choice_colors(n):
    if n > len(COLOR_PALETTE):
        return _colorline[0:-1:len(_colorline)/(n+1)]
    elif n:
        return COLOR_PALETTE[:n]
    return []

class TinyEvent(tg.widgets.Widget, interface.TinyWidget):
    
    template = """<div xmlns:py="http://purl.org/kid/ns#" class="calEvent" style="background-color: ${color}"
    starts="${str(starts)}" ends="${str(ends)}" record_id="${record_id}" py:content="title" title="${description}">
    </div>
    """

    params = ['starts', 'ends', 'title', 'description', 'color', 'record_id']

    starts = None
    ends = None
    dayspan = 0
    color = None
    
    title = ''
    description = ''

    record = {}
    record_id = False
    
    def __init__(self, record, starts, ends, title='', description='', dayspan=0, color=None):
        
        self.record = record
        self.record_id = record['id']
        
        if starts and ends:

            self.starts = (starts or None) and datetime.datetime(*starts[:6])
            self.ends = (ends or None) and datetime.datetime(*ends[:6])

        self.dayspan = dayspan
                    
        self.title = title
        self.description = description
        self.color = color

class ICalendar(interface.TinyCompoundWidget):
    """ Base Calendar calss
    """
    
    mode = 'month'
    date_start = None
    date_delay = None
    date_stop = None
    color_field = None
    day_length = 8
    use_search = False
    selected_day = None    
    date_format = '%Y-%m-%d'
    
    member_widgets = ['concurrency_info']
    css = [tg.widgets.CSSLink('openerp', 'tinycalendar/css/calendar.css')]
    javascript = [tg.widgets.JSLink('openerp', 'tinycalendar/javascript/calendar_date.js'),
                  tg.widgets.JSLink('openerp', 'tinycalendar/javascript/calendar_utils.js'),
                  tg.widgets.JSLink('openerp', 'tinycalendar/javascript/calendar_box.js'),
                  tg.widgets.JSLink('openerp', 'tinycalendar/javascript/calendar_month.js'),
                  tg.widgets.JSLink('openerp', 'tinycalendar/javascript/calendar_week.js')]

    def __init__(self, model, ids, view, domain=[], context={}, options=None):
        
        self.info_fields = []
        self.fields = []
        
        self.events = {}
    
        self.colors = {}
        self.color_values = []
        
        self.calendar_fields = {}
        self.concurrency_info = None

        self.ids = ids
        self.model = model        
        self.domain = domain or []
        self.context = context or {}
        self.options = options
                
        self.date_format = format.get_datetime_format('date')                
        self.use_search = (options or None) and options.use_search 
        
        try:            
            dt = parse_datetime(options.selected_day)
            self.selected_day = Day(dt.year, dt.month, dt.day)
        except:
            pass

        proxy = rpc.RPCProxy(model)

        view_id = view.get('view_id', False)

        dom = xml.dom.minidom.parseString(view['arch'].encode('utf-8'))
        root = dom.childNodes[0]
        attrs = tools.node_attributes(root)
        
        self.string = attrs.get('string', '')
        self.date_start = attrs.get('date_start') 
        self.date_delay = attrs.get('date_delay')
        self.date_stop = attrs.get('date_stop')
        self.color_field = attrs.get('color')
        self.day_length = int(attrs.get('day_length', 8))

        if options and options.mode:
            self.mode = options.mode
        else:
            self.mode = attrs.get('mode') or self.mode or 'month'

        self.info_fields = self.parse(root, view['fields'])
        
        fields = view['fields']
        fields = fields.keys() + [self.date_start, self.date_stop, self.date_delay, self.color_field]

        fields = list(set([x for x in fields if x]))

        self.fields = proxy.fields_get(fields)
        
        if self.color_field and options and options.colors:
            self.colors = options.colors
            
        if self.color_field and options and options.color_values:
            self.color_values = options.color_values
            
        self.calendar_fields['date_start'] = dict(name=self.date_start, 
                                                  kind=self.fields[self.date_start]['type'])
        
        if self.date_delay:
            self.calendar_fields['date_delay'] = dict(name=self.date_delay, 
                                                      kind=self.fields[self.date_delay]['type'])
            
        if self.date_stop:
            self.calendar_fields['date_stop'] = dict(name=self.date_stop, 
                                                         kind=self.fields[self.date_stop]['type'])
            
        self.calendar_fields['day_length'] = self.day_length

    def parse(self, root, fields):
        """ Deraived class must override parse method
        """
        pass
   
    def convert(self, event):
        
        fields = [x for x in [self.date_start, self.date_stop] if x]
        
        for fld in fields:
            typ = self.fields[fld]['type']
            assert typ in ('date', 'datetime'), "Invalid field type (%s), should be either `date` or `datetime`: %s" % (typ, fld)
            fmt = format.DT_SERVER_FORMATS[typ]

            if event[fld] and fmt:
                event[fld] = time.strptime(event[fld], fmt)
            
            # default start time is 9:00 AM
            if typ == 'date' and fld == self.date_start:
                ds = list(event[fld])
                ds[3] = 9
                event[fld] = tuple(ds)
   
    def get_events(self, days):
                    
        proxy = rpc.RPCProxy(self.model)
        
        #XXX: how to get events not falling between the given day range but spans the range?            
        #domain = self.domain + [(self.date_start, '>', days[0].prev().isoformat()), 
        #                        (self.date_start, '<', days[-1].next().isoformat())]
        
        first = days[0].month2.prev()[0] #HACK: add prev month
        domain = self.domain + [(self.date_start, '>', first.isoformat()), 
                                (self.date_start, '<', days[-1].next().isoformat())]
        
        # convert color values from string to python values
        if self.color_values and self.color_field in self.fields:
            try:
                import openerp.widgets as tw
                atr = self.fields[self.color_field]
                atr['required'] = False
                wid = tw.form.WIDGETS[atr['type']](atr)
                vals = self.color_values[:]
                for i, v in enumerate(vals):
                    try:
                        vals[i] = wid.validator.to_python(v)
                    except:
                        pass
                domain += [(self.color_field, "in", vals)]
            except Exception, e:
                pass

        if self.options and self.options.use_search:
            domain += self.options.search_domain
            
        ctx = rpc.session.context.copy()
        ctx.update(self.context)

        order_by = ('sequence' in self.fields or 0) and 'sequence'

        ids = proxy.search(domain, 0, 0, order_by, ctx)
        result = proxy.read(ids, self.fields.keys()+['__last_update'], ctx)
        self._update_concurrency_info(self.model, result)
        self.concurrency_info = interface.ConcurrencyInfo(self.model, ids)
        
        if self.color_field:
            
            for evt in result:   
                key = evt[self.color_field]
                name = key
                value = key
                
                if isinstance(key, (tuple, list)): #M2O 
                    value, name = key

                self.colors[key] = (name, value, None)

            colors = choice_colors(len(self.colors))
            for i, (key, value) in enumerate(self.colors.items()):
                self.colors[key] = (value[0], value[1], colors[i])
        
        events = []

        for evt in result:
            self.convert(evt)
            events += [self.get_event_widget(evt)]
        
        # filter out the events which are not in the range
        result = []
        for e in events:
            if e.dayspan > 0 and days[0] - e.dayspan < e.starts:
                result += [e]
            if e.dayspan == 0 and days[0] <= e.starts:
                result += [e]
        
        return result
    
    def get_event_widget(self, event):

        title = ''       # the title
        description = [] # the description
        starts = None    # the starting time (datetime)
        ends = None      # the end time (datetime)
        
        if self.info_fields:
            
            f = self.info_fields[0]
            s = event[f]
            
            if isinstance(s, (tuple, list)): s = s[-1]
            
            title = ustr(s)
        
            for f in self.info_fields[1:]:
                s = event[f]
                if isinstance(s, (tuple, list)): s = s[-1]
            
                description += [ustr(s)]

        starts = event.get(self.date_start)
        ends = event.get(self.date_delay) or 1.0
        span = 0
        
        if starts and ends:
            
            n = 0
            h = ends
            
            if ends == self.day_length: span = 1
            
            if ends > self.day_length:
                n = ends / self.day_length
                h = ends % self.day_length
            
                n = int(math.floor(n))
            
                if n > 0: span = n + 1
            
            ends = time.localtime(time.mktime(starts) + (h * 60 * 60) + (n * 24 * 60 * 60))
        
        if starts and self.date_stop:
            
            ends = event.get(self.date_stop)
            if not ends:
                ends = time.localtime(time.mktime(starts) + 60 * 60)
                
            tds = time.mktime(starts)
            tde = time.mktime(ends)
            
            if tds >= tde:
                tde = tds + 60 * 60
                ends = time.localtime(tde)
            
            n = (tde - tds) / (60 * 60)
            
            if n > self.day_length:
                span = math.floor(n / 24)

        starts = format.format_datetime(starts, "datetime", True)
        ends = format.format_datetime(ends, "datetime", True)
        
        color_key = event.get(self.color_field) 
        color = self.colors.get(color_key)

        title = title.strip()
        description = ', '.join(description).strip()
        
        return TinyEvent(event, starts, ends, title, description, dayspan=span, color=(color or None) and color[-1])


class TinyCalendar(ICalendar):

    def parse(self, root, fields):
        
        info_fields = []
        attrs = tools.node_attributes(root)        

        for node in root.childNodes:
            attrs = tools.node_attributes(node)
            
            if node.localName == 'field':
                info_fields += [attrs['name']]

        return info_fields

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

"""
This module implementes the RootController of the TurboGears application.
For more information on TG controllers, please see the TG docs.
"""

import os

from turbogears import controllers
from turbogears import expose
from turbogears import redirect
from turbogears import config

import cherrypy

from openerp import rpc
from openerp import common
from openerp import stdvars

from openerp import subcontrollers
from openerp import cache

from openerp.tinyres import TinyResource, unsecured

import pkg_resources
from turbogears.widgets import register_static_directory

treegrid_static_dir = pkg_resources.resource_filename("openerp",  "static")
register_static_directory("openerp", treegrid_static_dir)

config.update({'i18n.gettext' : cache.gettext})

# initialize the rpc session
host = config.get('host', path="openerp")
port = config.get('port', path="openerp")
protocol = config.get('protocol', path="openerp")
rpc.session = rpc.RPCSession(host, port, protocol, storage=cherrypy.session)

class Root(controllers.RootController, TinyResource):
    """Turbogears root controller, see TG docs for more info.
    """

    def user_action(self, id='action_id'):
        """Perform default user action.

        @param id: `action_id` or `menu_id`
        """

        proxy = rpc.RPCProxy("res.users")
        act_id = proxy.read([rpc.session.uid], [id, 'name'], rpc.session.context)

        if not act_id[0][id]:
            common.warning(_('You can not log into the system!\nAsk the administrator to verify\nyou have an action defined for your user.'), _('Access Denied!'))
            rpc.session.logout()
            raise redirect('/');

        act_id = act_id[0][id][0]

        return subcontrollers.actions.execute_by_id(act_id)

    @expose()
    def index(self):
        """Index page, loads the view defined by `action_id`.
        """
        return self.user_action('action_id')

    @expose()
    def menu(self):
        """Main menu page, loads the view defined by `menu_id`.
        """
        return self.user_action('menu_id')

    def _cp_on_error(self, *args, **kw):
        message = self.errorpage.render()
        cherrypy.response.headers['Content-Type'] = 'text/html'
        cherrypy.response.body = [message]

    @expose(template="openerp.templates.login")
    @unsecured
    def login(self, db=None, user=None, password=None):

        message = None

        url = rpc.session.get_url()
        dblist = rpc.session.listdb()
        
        if dblist == -1:
            dblist = []
            message = _("Could not connect to server!")

        return dict(target='/', url=url, dblist=dblist, user=user, password=password, db=db, action='login', message=message, origArgs={})

    @expose()
    @unsecured
    def get_logo(self):          
        
        comp_url = config.get('company.url', path='openerp-web') or None
        
        res="""<img src="/static/images/openerp_big.png" alt="%(alt)s" border="0" width="200px" height="60px" usemap="#logo_map"/>
                    <map name="logo_map">
                        <area shape="rect" coords="102,42,124,56" href="http://openerp.com" target="_blank"/>
                        <area shape="rect" coords="145,42,184,56" href="http://axelor.com" target="_blank"/>
                    </map>"""%(dict(alt=_('OpenERP')))
                    
        if os.path.exists(pkg_resources.resource_filename("openerp", "static/images/company_logo.png")):
            if comp_url:
                res = """   <a href='"""+comp_url+"""' target='_blank'>
                                <img src='/static/images/company_logo.png' alt="" border="0" width="205px" height="58px"/> 
                            </a> """
            else:
                 res = """<img src="/static/images/company_logo.png" alt="" border="0" width="205px" height="58px"/>"""
        return res
    
    @expose()
    @unsecured
    def developped_by(self):
        comp_url = config.get('company.url', path='openerp-web') or None
        
        res="""<img src="/static/images/developped_by.png" border="0" width="200" height="60" alt="%(alt)s" usemap="#devby_map"/>
                    <map name="devby_map">
                        <area shape="rect" coords="0,20,100,60" href="http://axelor.com" target="_blank"/>
                        <area shape="rect" coords="120,20,200,60" href="http://openerp.com" target="_blank"/>
                    </map>"""%(dict(alt=_('Developped by Axelor and Tiny')))
                    
        if os.path.exists(pkg_resources.resource_filename("openerp", "static/images/company_logo.png")):
            if comp_url:
                res = """   <a href='"""+comp_url+"""' target='_blank'>
                                <img src='/static/images/company_logo.png' alt="" border="0" width="205px" height="58px"/> 
                            </a> """
            else:
                 res = """<img src="/static/images/company_logo.png" alt="" border="0" width="205px" height="58px"/>"""
        return res
    
    @expose()
    @unsecured
    def logout(self):
        """ Logout method, will terminate the current session.
        """
        rpc.session.logout()
        raise redirect('/')

    @expose(template="openerp.templates.about")
    @unsecured
    def about(self):
        from openerp import release
        version = _("Version %s-%s") % (release.version, release.release)
        return dict(version=version)

    form = subcontrollers.form.Form()
    tree = subcontrollers.tree.Tree()
    graph = subcontrollers.graph.Graph()
    wizard = subcontrollers.wizard.Wizard()
    search = subcontrollers.search.Search()
    pref = subcontrollers.preferences.Preferences()
    selection = subcontrollers.selection.Selection()
    shortcuts = subcontrollers.shortcuts.Shortcuts()
    requests = subcontrollers.requests.Requests()
    openm2o = subcontrollers.openm2o.OpenM2O()
    openo2m = subcontrollers.openo2m.OpenO2M() 
    openm2m = subcontrollers.openm2m.OpenM2M()   
    listgrid = subcontrollers.listgrid.List()
    attachment = subcontrollers.attachment.Attachment()
    translator = subcontrollers.translator.Translator()
    impex = subcontrollers.impex.ImpEx()
    fieldpref = subcontrollers.fieldpref.FieldPref()
    calendar = subcontrollers.tinycalendar.TinyCalendar()
    calpopup = subcontrollers.tinycalendar.CalendarPopup()
    viewlog = subcontrollers.view_log.View_Log()
    image = subcontrollers.image.Image()    
    database = subcontrollers.database.Database()
    viewed = subcontrollers.viewed.ViewEd()
    viewlist = subcontrollers.viewlist.ViewList()
    workflow = subcontrollers.workflow.Workflow()
    workflowlist = subcontrollers.workflow.WorkflowList()
    process = subcontrollers.process.Process()
    wiki = subcontrollers.wiki.WikiView()
    errorpage = subcontrollers.error_page.ErrorPage()

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

from turbogears import view
from openerp import rpc

def tg_query(*args, **kw):
    """Returns url querrystring from the provided arguments...
    for example:

    >>> print tg_query('/graph', 'pie', width=100, height=100)
    >>> "/graph/pie?width=100&height=100"

    """
    result = []
    for k, v in kw.items():
        result += ['%s=%s'%(k, v)]

    url = '/'.join([ustr(a) for a in args])
    return ((url or '') and url + '?') + '&'.join(result)

def add_root_vars(root_vars):
    return root_vars.update({'rpc': rpc})

def add_vars(vars):
    from cherrypy import root

    std_vars = {
        'root': root,
        'query': tg_query
    }

    return vars.update(std_vars)

view.root_variable_providers.append(add_root_vars)
view.variable_providers.append(add_vars)

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

# Release information about openerp-web

version = "5.0.0"
release = "3.1"

description = "Web Client of OpenERP, the Enterprise Management Software"
long_description = "OpenERP Web is the web client of the OpenERP, a free enterprise management software: accounting, stock, manufacturing, project mgt..."
author = "Tiny ERP Pvt. Ltd."
email = "info@openerp.com"
copyright = "Copyright (C) 2007-TODAY Tiny ERP Pvt. Ltd."

# if it's open source, you might want to specify these
url = "http://openerp.com/"
download_url = "http://www.openerp.com/index.php?option=com_content&view=article&id=18&Itemid=28"
license = "OEPL"

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

from turbogears import widgets

from interface import TinyCompoundWidget

class Pager(TinyCompoundWidget):

    template = "openerp.widgets.templates.pager"
    params = ['offset', 'limit', 'count', 'prev', 'next', 'page_info', 'pager_id']

    css = [widgets.CSSLink('openerp', 'css/pager.css')]

    offset = 0
    limit = 20
    count = 0

    page_info = None
    pager_id = 1

    def __init__(self, id=False, ids=[], offset=0, limit=20, count=0, view_type='tree'):

        super(Pager, self).__init__()

        self.limit = limit or 20
        self.offset = offset or 0
        self.count = count

        self.id = id or False
        self.ids = ids or []
        
        if len(self.ids) > self.limit:
            self.ids = self.ids[self.offset:]
            self.ids = self.ids[:min(self.limit, len(self.ids))]

        if view_type == 'form':

            index = 0
            if self.id in self.ids:
                index = self.offset + self.ids.index(self.id) + 1

            self.page_info = _("[%s/%s]") % (index or '-', self.count)

            self.prev = index > 1
            self.next = index < self.count

        else:
            index = (self.count or 0) and self.offset + 1
            
            o = self.offset + len(self.ids)
            o = min(self.count, o)

            self.page_info = _("[%s - %s of %s]") % (index, o, self.count)
            self.prev = self.offset > 0
            self.next = self.offset+len(self.ids) < self.count

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import os
import base64

from turbogears import expose
from turbogears import controllers
from turbogears import redirect

from openerp import rpc
from openerp.tinyres import TinyResource
from openerp.utils import TinyDict

import openerp.widgets as tw

class ViewList(controllers.Controller, TinyResource):

    @expose(template="openerp.subcontrollers.templates.viewlist")
    def index(self, model):
        
        params = TinyDict()
        params.model = 'ir.ui.view'
        params.view_mode = ['tree']
        
        params.domain = [('model', '=', model)]
        
        screen = tw.screen.Screen(params, selectable=1)
        screen.widget.pageable = False
        
        return dict(screen=screen, model=model, show_header_footer=False)
    
    @expose()
    def create(self, model, **kw):
        
        view_name = kw.get('name')
        view_type = kw.get('type')
        priority = kw.get('priority', 16)
        
        if not view_name:
            raise redirect('/viewlist', model=model)

        proxy = rpc.RPCProxy(model)
        fields = proxy.fields_get({}).keys()
        string = "Unknown"

        try:
            proxy2 = rpc.RPCProxy('ir.model')
            mid = proxy2.search([('model','=',model)])[0]
            string = proxy2.read([mid], ['name'])[0]['name']
        except:
            pass

        fname = None
        for n in ('name', 'x_name'):
            if n in fields:
                fname = n
                break

        if fname:
            arch = """<?xml version="1.0"?>
            <%s string="%s">
                <field name="%s"/>
            </%s>
            """ % (view_type, string, fname, view_type)

            proxy = rpc.RPCProxy('ir.ui.view')
            proxy.create(dict(model=model, name=view_name, type=view_type, priority=priority, arch=arch))
        
        raise redirect('/viewlist', model=model)
    
    @expose()
    def delete(self, model, id):
        
        id = int(id)
        
        proxy = rpc.RPCProxy('ir.ui.view')
        proxy.unlink(id)
        
        raise redirect('/viewlist', model=model)

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

from turbogears import expose
from turbogears import widgets
from turbogears import validators
from turbogears import validate
from turbogears import error_handler
from turbogears import exception_handler

import cherrypy

from openerp import rpc
from openerp import cache
from openerp import tools
from openerp import widgets as tw

from openerp.utils import TinyDict

from form import Form
from form import get_validation_schema
from form import default_error_handler
from form import default_exception_handler

class OpenO2M(Form):
    
    path = '/openo2m'    # mapping from root
    
    def create_form(self, params, tg_errors=None):
        
        params.id = params.o2m_id
        params.model = params.o2m_model        
        params.view_mode = ['form', 'tree']
        params.view_type = 'form'

        # to get proper view, first generate form using the view_params
        vp = params.view_params
        form = tw.form_view.ViewForm(vp, name="view_form", action="/form/save")
        cherrypy.request.terp_validators = {}
        wid = form.screen.widget.get_widgets_by_name(params.o2m)[0]

        # save view_params for later phazes
        vp = vp.make_plain('_terp_view_params/')
        hiddens = map(lambda x: widgets.HiddenField(name=x, default=ustr(vp[x])), vp)

        params.prefix = params.o2m
        params.views = wid.view

        ctx = params.context or {}
        ctx.update(params.parent_context or {})
        ctx.update(params.o2m_context or {})
        p, ctx = TinyDict.split(ctx)
        
        params.context = ctx or {}

        form = tw.form_view.ViewForm(params, name="view_form", action="/openo2m/save")
        form.hidden_fields = [widgets.HiddenField(name='_terp_parent_model', default=params.parent_model),
                              widgets.HiddenField(name='_terp_parent_id', default=params.parent_id),
                              widgets.HiddenField(name='_terp_parent_context', default=ustr(params.parent_context)),
                              widgets.HiddenField(name='_terp_o2m', default=params.o2m),                              
                              widgets.HiddenField(name='_terp_o2m_id', default=params.id or None),
                              widgets.HiddenField(name='_terp_o2m_model', default=params.o2m_model),
                              widgets.HiddenField(name='_terp_o2m_context', default=ustr(params.o2m_context or {})),
                              widgets.HiddenField(name=params.prefix + '/__id', default=params.id or None)] + hiddens

        return form
    
    @expose(template="openerp.subcontrollers.templates.openo2m")
    def create(self, params, tg_errors=None):

        if tg_errors:
            form = cherrypy.request.terp_form
        else:
            form = self.create_form(params, tg_errors)        
                
        return dict(form=form, params=params, show_header_footer=False)
    
    @expose()
    @validate(form=get_validation_schema)
    @error_handler(default_error_handler)
    @exception_handler(default_exception_handler)
    def save(self, terp_save_only=False, **kw):
        params, data = TinyDict.split(kw)
        params.editable = True

        proxy = rpc.RPCProxy(params.parent_model)

        pprefix = '.'.join(params.o2m.split('/')[:-1])

        if pprefix:
            data = eval(pprefix, TinyDict(**data)).make_dict()

        ctx = tools.context_with_concurrency_info(rpc.session.context, params.concurrency_info)
        ctx.update(params.parent_context or {})
        ctx.update(params.o2m_context or {})

        id = proxy.write([params.parent_id], data, ctx)

        # perform button action
        if params.button:
            res = self.button_action(params)
            if res:
                return res

        prefix = params.o2m
        current = params.chain_get(prefix)

        params.load_counter = 1

        if current and current.id and not params.button:
            params.load_counter = 2

        # If new O2M action, get the newly created id
        if params.source:
            field = prefix.split('/')[-1]
            current.id = proxy.read([params.parent_id], [field], ctx)[0][field][0]
            params.o2m_id = current.id

        return self.create(params)
    
    @expose()
    def edit(self, **kw):
        params, data = TinyDict.split(kw)
        return self.create(params)

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import time
import datetime
import calendar
import xml.dom.minidom

import cherrypy
import turbogears as tg

from openerp import rpc
from openerp import tools
from openerp.utils import TinyDict

from openerp.widgets import interface

from base import ICalendar
from base import TinyCalendar

from utils import Day
from utils import Week
from utils import Month
from utils import Year

class MiniCalendar(tg.widgets.CompoundWidget, interface.TinyWidget):
    template = 'openerp.widgets.tinycalendar.templates.mini'
    params = ['selected_day', 'month', 'forweek', 'highlight']
    
    month = None
    selected_day = None
    forweek = False
    highlight = True
        
    def __init__(self, selected_day, forweek=False, highlight=True):        
        self.month = Month(selected_day.year, selected_day.month)
        self.selected_day = selected_day
        self.forweek = forweek
        self.highlight = highlight
        
class GroupBox(tg.widgets.CompoundWidget, interface.TinyWidget):
    template = 'openerp.widgets.tinycalendar.templates.groups'
    params = ["colors", "color_values", "title"]
        
    colors = {}
    color_values = []
    title = None
    action = None
        
    def __init__(self, colors, color_values, selected_day, title=None, mode='month'):
        self.colors = colors
        self.color_values = color_values
        self.title = title
        
def get_calendar(model, view, ids=None, domain=[], context={}, options=None):
        
    mode = (options or None) and options.mode
    if not mode:
        dom = xml.dom.minidom.parseString(view['arch'].encode('utf-8'))
        attrs = tools.node_attributes(dom.childNodes[0])
        mode = attrs.get('mode')
    
    if mode == 'day':
        return DayCalendar(model, view, ids, domain, context, options)
        
    if mode == 'week':
        return WeekCalendar(model, view, ids, domain, context, options)
    
    return MonthCalendar(model, view, ids, domain, context, options)

def _get_selection_day(day, selected, mode):
    selected = selected or Day.today()

    if mode == 'day':
        return day

    if mode == 'week':
        return Week(day)[selected.weekday()]

    month = day.month2
    d = selected.day

    if d > month.range[-1]:
        d = month.range[-1]

    return Day(day.year, day.month, d)

class MonthCalendar(TinyCalendar):

    template = 'openerp.widgets.tinycalendar.templates.month'
    params = ['month', 'events', 'selected_day', 'calendar_fields', 'date_format']
    member_widgets = ['minical', 'groupbox', 'use_search']    

    month = None
    events = {}
    
    minical = None
    groupbox = None    

    def __init__(self, model, view, ids=None, domain=[], context={}, options=None):
        
        TinyCalendar.__init__(self, model, ids, view, domain, context, options)                
        
        y, m = time.localtime()[:2]
        if options:
            y = options.year
            m = options.month

        self.month = Month(y, m)
        self.events = self.get_events(self.month.days)

        self.selected_day = _get_selection_day(Day(y, m, 1), self.selected_day, 'month')

        self.minical = MiniCalendar(self.selected_day)            
        self.groupbox = GroupBox(self.colors, self.color_values, self.selected_day, 
                title=(self.color_field or None) and self.fields[self.color_field]['string'], 
                mode='month')

            
class WeekCalendar(TinyCalendar):
    template = 'openerp.widgets.tinycalendar.templates.week'
    params = ['week', 'events', 'selected_day', 'calendar_fields', 'date_format']
    member_widgets = ['minical', 'groupbox', 'use_search']
    
    week = None
    events = {}
    
    minical = None
         
    def __init__(self, model, view, ids=None, domain=[], context={}, options=None):            
        TinyCalendar.__init__(self, model, ids, view, domain, context, options)

        y, m, d = time.localtime()[:3]
        if options:
            y, m, d = options.date1[:3]
        
        self.week = Week(Day(y, m, d))
        self.events = self.get_events(self.week.days)

        self.selected_day = _get_selection_day(Day(y, m, d), self.selected_day, 'week')

        self.minical = MiniCalendar(self.week[0], True)
        self.groupbox = GroupBox(self.colors, self.color_values, self.week[0], 
                title=(self.color_field or None) and self.fields[self.color_field]['string'], 
                mode='week')

class DayCalendar(TinyCalendar):
    template = 'openerp.widgets.tinycalendar.templates.day'
    params = ['day', 'events', 'calendar_fields', 'date_format']
    member_widgets = ['minical', 'groupbox', 'use_search']
    
    day = None
    events = {}
    
    minical = None
         
    def __init__(self, model, view, ids=None, domain=[], context={}, options=None):            
        TinyCalendar.__init__(self, model, ids, view, domain, context, options)
        
        y, m, d = time.localtime()[:3]
        if options:
            y, m, d = options.date1[:3]
                     
        self.day = Day(y,m,d)

        self.events = self.get_events([self.day])        
        self.minical = MiniCalendar(self.day)
        self.groupbox = GroupBox(self.colors, self.color_values, self.day, 
                title=(self.color_field or None) and self.fields[self.color_field]['string'], 
                mode='day')

class GanttCalendar(ICalendar):
    
    template = 'openerp.widgets.tinycalendar.templates.gantt'

    params = ['title', 'level', 'groups', 'days', 'events', 'calendar_fields', 'date_format', 
              'selected_day', 'mode', 'headers', 'subheaders', 'model', 'ids']
    member_widgets = ['groupbox', 'use_search', 'extra_css', 'extra_javascript']

    level = None
    groups = None
    title = None
    days = None
    headers = None
    subheaders = None
    mode = 'week'

    extra_css = [tg.widgets.CSSLink('openerp', 'tinycalendar/css/calendar_gantt.css')]
    extra_javascript = [tg.widgets.JSLink('openerp', 'tinycalendar/javascript/calendar_gantt.js')]

    def __init__(self, model, ids, view, domain=[], context={}, options=None):

        self.level = None
        self.groups = []
        self.days = []
        self.headers = []

        super(GanttCalendar, self).__init__(model, ids, view, domain, context, options)

        y, m, d = time.localtime()[:3]
        if options:
            y, m, d = options.date1[:3]

        day = Day(y, m, d)

        if self.mode == 'day':
            self.days = [day]
            self.title = ustr(day)
            self.selected_day = day

            self.headers = [(48, ustr(day))]
            self.subheaders = [time.strftime('%I %P', (y, m, d, i, 0, 0, 1, 1, 0)) for i in range(24)]

        elif self.mode == '3days':
            dp = day - 1
            dn = day + 1
            self.days = [dp, day, dn]
            self.title = u"%s, %s, %s" % (ustr(dp), ustr(day), ustr(dn))
            self.selected_day = day

            self.headers = [(24, ustr(dp)), (24, ustr(day)), (24, ustr(dn))]
            self.subheaders = []
            for x in self.headers:
                self.subheaders += [time.strftime('%I %P', (y, 1, 1, i, 0, 0, 1, 1, 0)) for i in range(0, 24, 6)]

        elif self.mode == 'week':
            self.days = [d for d in Week(day)]
            self.title = _("%s, Week %s") % (y, day.strftime("%W"))
            self.selected_day = _get_selection_day(day, self.selected_day, 'week')
            self.headers = [(12, u"%s %s" % (d.month2.name, d.day)) for d in self.days]
            self.subheaders = []
            for x in self.days:
                self.subheaders += [time.strftime('%I %P', (y, 1, 1, i, 0, 0, 1, 1, 0)) for i in [0, 23]]

        elif self.mode == '3weeks':
            w = Week(day)
            wp = w - 1
            wn = w + 1
            self.days = wp.days + w.days + wn.days
            self.title = _(u"%s - %s") % (ustr(self.days[0]), ustr(self.days[-1]))
            self.selected_day = _get_selection_day(day, self.selected_day, 'week')
            self.headers = [(7, _("Week %s") % w1[0].strftime('%W')) for w1 in [wp, w, wn]]
            self.subheaders = []
            self.subheaders += [x.strftime('%a %d') for x in wp]
            self.subheaders += [x.strftime('%a %d') for x in w]
            self.subheaders += [x.strftime('%a %d') for x in wn]

        elif self.mode == '3months':
            q = 1 + (m - 1) / 3

            mn = Month(y, q * 3)
            mt = mn.prev()
            mp = mt.prev()

            days = []
            days += [d for d in mp if d.year == mp.year and d.month == mp.month]
            days += [d for d in mt if d.year == mt.year and d.month == mt.month]
            days += [d for d in mn if d.year == mn.year and d.month == mn.month]

            self.days = days
            self.title = _("%s, Qtr %s") % (y, q)
            self.selected_day = _get_selection_day(day, self.selected_day, '3months')
            
            headers = []
            headers += [w for w in mp.weeks]
            headers += [w for w in mt.weeks]
            headers += [w for w in mn.weeks]

            self.headers = [(mp.range[-1], ustr(mp)), (mt.range[-1], ustr(mt)), (mn.range[-1], ustr(mn))]
            self.subheaders = []
            for x in headers:
                x = _("Week %s") % x[0].strftime('%W')
                if x not in self.subheaders:
                    self.subheaders += [x]

        elif self.mode == 'year':
            yr = Year(y)

            self.days = yr.days
            self.title = u"Year %s" % (y)
            self.selected_day = _get_selection_day(day, self.selected_day, 'year')
            self.headers = [(m.range[-1], m.name) for m in yr.months]
            self.subheaders = [_("W %s") % x[0].strftime('%W') for x in yr.weeks]

        elif self.mode == '3years':
            yr = Year(y)
            yp = yr - 1
            yn = yr + 1

            self.days = yp.days + yr.days + yn.days
            self.title = _("Year %s to Year %s") % (y - 1, y + 1)
            self.selected_day = _get_selection_day(day, self.selected_day, 'year')

            self.headers = [(4, y - 1), (4, y), (4, y + 1)]
            self.subheaders = ['Q1', 'Q2', 'Q3', 'Q4'] * 3

        elif self.mode == '5years':
            yr = Year(y)
            yp1 = yr - 1
            yp2 = yr - 2
            yn1 = yr + 1
            yn2 = yr + 1

            self.days = yp2.days + yp1.days + yr.days + yn1.days + yn2.days
            self.title = _("Year %s to Year %s") % (y - 2, y + 2)
            self.selected_day = _get_selection_day(day, self.selected_day, 'year')

            self.headers = [(2, y - 2), (2, y - 1), (2, y), (2, y + 1), (2, y + 2)]
            self.subheaders = ['H1', 'H2'] * 5

        else:
            month = Month(y, m)
            self.days = [d for d in month]
            self.title = ustr(month)
            self.selected_day = _get_selection_day(day, self.selected_day, 'month')
            self.headers = [(7, _("Week %s") % w[0].strftime('%W')) for w in month.weeks]
            self.subheaders = [d.day for d in month]

        if self.level:
            field = self.level['link']
            fields = rpc.RPCProxy(self.model).fields_get([field])
            self.fields.update(fields)

        self.events = self.get_events(self.days)
        self.groups = self.get_groups(self.events)
        self.groupbox = GroupBox(self.colors, self.color_values, day, 
                title=(self.color_field or None) and self.fields[self.color_field]['string'], mode=self.mode)

    def parse(self, root, fields):
        
        info_fields = []
        attrs = tools.node_attributes(root)

        for node in root.childNodes:
            attrs = tools.node_attributes(node)
            
            if node.localName == 'field':
                info_fields += [attrs['name']]

            if node.localName == 'level':
                self.level = attrs
                info_fields += self.parse(node, fields)

        return info_fields

    def get_groups(self, events):

        if not self.level:
            return []

        obj = self.level['object']
        field = self.level['link']
        domain = self.level['domain']

        keys = []
        groups = {}
        for evt in events:
            group_id = evt.record[field]
            group_title = 'None'

            if not group_id: # create dummy group
                group_id = 0

            if isinstance(group_id, (list, tuple)):
                group_id, group_title = evt.record[field]
            elif group_id:
                group_id, group_title = rpc.RPCProxy(obj).name_get([group_id], rpc.session.context)[0]

            group = groups.setdefault(group_id, {'id': group_id, 'title': group_title, 'model': obj, 'items': []})

            group['items'].append(evt.record_id)

            if group_id not in keys:
                keys.append(group_id)

        return [groups[i] for i in keys]

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import os
import base64

from turbogears import expose
from turbogears import controllers

from openerp import rpc
from openerp.tinyres import TinyResource
from openerp.utils import TinyDict
from openerp.subcontrollers import actions
from openerp import common

import openerp.widgets as tw

class Attachment(controllers.Controller, TinyResource):

    @expose()
    def index(self, model, id):

        id = int(id)
        
        if id:
            ctx = {}
            ctx.update(rpc.session.context.copy())

            action = rpc.session.execute('object', 'execute', 'ir.attachment', 'action_get', ctx)

            action['domain'] = [('res_model', '=', model), ('res_id', '=', id)]
            ctx['default_res_model'] = model
            ctx['default_res_id'] = id
            action['context'] = ctx
            
            return actions.execute(action) 
        else:
            raise common.message(_('No record selected! You can only attach to existing record...'))
        
        return True
    
    @expose(content_type="application/octet-stream")
    def save_as(self, fname=None, record=False, **kw):
        record = int(record)
        proxy = rpc.RPCProxy('ir.attachment')
        
        data = proxy.read([record], [], rpc.session.context)
        if len(data) and not data[0]['link'] and data[0]['datas']:
            return base64.decodestring(data[0]['datas'])
        else:
            return '' 

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import sys
import cgitb

from turbogears import expose
from turbogears import redirect
from turbogears import controllers

import cherrypy

from openerp import rpc
from openerp import tools
from openerp import common

import openerp.widgets as tw

class ErrorPage(controllers.Controller):

    nb = tw.form.Notebook({}, [])

    @expose()
    def index(self, *args, **kw):
        raise redirect('/')

    def render(self):
        etype, value, tb = sys.exc_info()

        if not isinstance(value, common.TinyException):
            return cgitb.html((etype, value, tb))

        return self.__render(value)

    @expose(template="openerp.subcontrollers.templates.error_page")
    def __render(self, value):
        title=value.title
        error=value.message
        maintenance = None

        if isinstance(value, common.TinyError):
            proxy = rpc.RPCProxy('maintenance.contract')
            maintenance = proxy.status()

        return dict(title=title, error=error, maintenance=maintenance, nb=self.nb)

    @expose('json')
    def submit(self, tb, explanation, remarks):
        try:
            res = rpc.RPCProxy('maintenance.contract').send(tb, explanation, remarks)
            if res:
                return dict(message=_('Your problem has been sent to the quality team!\nWe will recontact you after analysing the problem.'))
            else:
                return dict(error=_('Your problem could not be sent to the quality team!\nPlease report this error manually at %s') % ('http://openerp.com/report_bug.html'))
        except Exception, e:
            return dict(error=str(e))

_ep = ErrorPage()

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import turbogears as tg
import cherrypy

from openerp import tools

def eval_get(attrs, name, default=None):
    if name not in attrs:
        return default

    val = attrs[name]
    if isinstance(val, basestring):
        val = val.title()

    return tools.expr_eval(val)

class TinyWidget(object):
    """Widget interface, every widget class should implement
    this class.
    """

    colspan = 1
    rowspan = 1
    string = None
    nolabel = False
    select = False
    required = False
    readonly = False
    help = None
    editable = True
    translatable = False
    visible = True
    inline = False

    name = None
    model = None
    states = None
    callback = None
    change_default = None
    onchange = 'onChange(this)'
    kind=None
    
    field_class = None

    def __init__(self, attrs={}):
        self.string = attrs.get("string", None)
        self.model = attrs.get("model", None)

        prefix = attrs.get('prefix', '')
        self.name = prefix + (prefix and '/' or '') + attrs.get('name', '')

        self.states = attrs.get('states', None)
        if isinstance(self.states, basestring):
            self.states = self.states.split(',')

        self.colspan = int(attrs.get('colspan', 1))
        self.rowspan = int(attrs.get('rowspan', 1))
        
        self.select = eval_get(attrs, 'select', False)
        self.nolabel = eval_get(attrs, 'nolabel', False)
        self.required = eval_get(attrs, 'required', False)
        self.readonly = eval_get(attrs, 'readonly', False)
        self.visible = True
        self.inline = attrs.get('inline');
        
        try:
            visval = attrs.get('invisible', 'False')
            ctx = attrs.get('context', {})
            self.invisible = eval(visval, {'context': ctx})
        except:
            pass
        
        self.help = attrs.get('help')
        self.editable = attrs.get('editable', True)
        self.translatable = attrs.get('translate', False)

        self.set_state(attrs.get('state', 'draft'))

        self.callback = attrs.get('on_change', None)
        self.change_default = attrs.get('change_default', False)
        self.kind = attrs.get('type', None)

        self.attributes = attrs.get('attrs', {})

    def set_state(self, state):

        if isinstance(self.states, dict) and state in self.states:

            attrs = dict(self.states[state])

            if 'readonly' in attrs:
                self.readonly = attrs['readonly']

            if 'required' in attrs:
                self.required = attrs['required']

            if 'value' in attrs:
                self.default = attrs['value']

class TinyInputWidget(TinyWidget):
    """Interface for Field widgets, every InputField widget should
    implement this class
    """

    def __init__(self, attrs={}):
        TinyWidget.__init__(self, attrs)
        self._validator = None

    def get_validator(self):

        # required fields are now validated at client side
        #if self._validator:
        #    self._validator.not_empty = (self.required or False) and True
        #elif self.required:
        #    self._validator = tg.validators.NotEmpty()

        return self._validator

    def set_validator(self, value):
        self._validator = value

    validator = property(get_validator, set_validator)

    def get_value(self):
        """Get the value of the field.

        @return: field value
        """
        return self.default

    def set_value(self, value):
        """Set the value of the field.

        @param value: the value
        """
        if isinstance(value, basestring):
            value = ustr(value)

        self.default = value
        
    def get_display_value(self):
        """Get the display value of the field.
        """
        
        try:
            return self.validator.from_python(self.default)
        except:
            pass
        
        return self.get_value()
    
    def update_params(self, d):
        super(TinyInputWidget, self).update_params(d)

        attrs = d['attrs'] = {}

        attrs['change_default'] = self.change_default or None
        attrs['callback'] = self.callback or None
        attrs['onchange'] = self.onchange

        # name as field_id
        d['field_id'] = self.name
        d['kind'] = self.kind
        d['editable'] = self.editable
        d['inline'] = self.inline

        if self.readonly:
            d['field_class'] = " ".join([d['field_class'], "readonlyfield"])
            attrs['disabled'] = 'disabled'

        if self.required and 'requiredfield' not in d['field_class'].split(' '):
            d['field_class'] = " ".join([d['field_class'], "requiredfield"])

        if self.translatable and 'translatable' not in d['field_class'].split(' '):
            d['field_class'] = " ".join([d['field_class'], "translatable"])

        if hasattr(self, 'error') and self.error:
            d['field_class'] = " ".join([d['field_class'], "errorfield"])

class TinyCompoundWidget(TinyInputWidget, tg.widgets.CompoundWidget):

    def __init__(self, attrs={}):
        TinyInputWidget.__init__(self, attrs)
        tg.widgets.CompoundWidget.__init__(self, name=self.name or None)

    def get_widgets_by_name(self, name, kind=tg.widgets.Widget, parent=None):
        result = []
        parent = parent or self

        for wid in parent.iter_member_widgets():

            if wid.name == name and isinstance(wid, kind):
                result.append(wid)

            if isinstance(wid, tg.widgets.CompoundWidget):
                result += self.get_widgets_by_name(name, kind=kind, parent=wid)

        return result

    def get_last_update_info(resource, values):
        result = {}
        for item in values:
            result["%s,%s" % (resource, item['id'])] = item.pop('__last_update', '')
        return result

    def _update_concurrency_info(self, resource, records):
        info = getattr(cherrypy.request, 'terp_concurrency_info', {})
        vals = info.setdefault(resource, {})
        for item in records:
            vals[item['id']] = item.pop('__last_update', '')
        cherrypy.request.terp_concurrency_info = info

    def update_params(self, d):
        tg.widgets.CompoundWidget.update_params(self, d)
        d['editable'] = self.editable
        
class TinyField(TinyInputWidget, tg.widgets.FormField):

    def __init__(self, attrs):
        TinyInputWidget.__init__(self, attrs)
        tg.widgets.FormField.__init__(self, name=self.name)

class ConcurrencyInfo(TinyCompoundWidget):
    template="""<span xmlns:py="http://purl.org/kid/ns#" py:strip="" py:if="ids and model in info">
        <input type="hidden" py:if="id in info[model]" py:for="id in ids"
            name="_terp_concurrency_info" value="('$model,$id', '${info[model][id]}')"/>
    </span>"""

    params = ['ids', 'model', 'info']

    def __init__(self, model, ids):
        self.ids = ids
        self.model = model

    def _get_concurrency_info(self):
        return getattr(cherrypy.request, 'terp_concurrency_info', {})

    info = property(_get_concurrency_info)

# vim: ts=4 sts=4 sw=4 si et





###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import os
import copy

from turbogears import expose
from turbogears import controllers

import cherrypy

from openerp import rpc
from openerp import tools
from openerp import common
from openerp import cache

from openerp.tinyres import TinyResource
from openerp.utils import TinyDict

import openerp.widgets as tw

class View_Log(controllers.Controller, TinyResource):

    @expose(template="openerp.subcontrollers.templates.view_log")
    def index(self, **kw):
        params, data = TinyDict.split(kw)
        
        id = params.id
        model = params.model
        message = None
        tmp = {}
        todo = []

        if id:
            res = rpc.session.execute('object', 'execute', model, 'perm_read', [id], rpc.session.context)

            for line in res:
                todo = [
                    ('id', _('ID')),
                    ('create_uid', _('Creation User')),
                    ('create_date', _('Creation Date')),
                    ('write_uid', _('Latest Modification by')),
                    ('write_date', _('Latest Modification Date')),
                    ('uid', _('Owner')),
                    ('gid', _('Group Owner')),
                    ('level', _('Access Level'))
                ]
                for (key,val) in todo:
                    if line.get(key) and key in ('create_uid','write_uid','uid'):
                        line[key] = line[key][1]
    
                    tmp[key] = ustr(line.get(key) or '/')
        
        if not id:
            message = _("No resource is selected...")

        return dict(tmp=tmp, todo=todo, message=message, show_header_footer=False)

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

from turbogears import controllers
from turbogears import expose
from turbogears import redirect

import cherrypy

from openerp import rpc
from openerp import common
from openerp import format

from openerp.utils import TinyDict
from openerp.tinyres import TinyResource

import form
import actions

class ResourcePopup(form.Form):
    
    path = '/process/resource'    # mapping from root
    
    @expose(template="openerp.subcontrollers.templates.process_open")
    def create(self, params, tg_errors=None):
        params.editable = True

        if params.id and cherrypy.request.path == self.path + '/view':
            params.load_counter = 2

        form = self.create_form(params, tg_errors)       
        return dict(form=form, params=params, show_header_footer=False)   

class Process(controllers.Controller, TinyResource):

    resource = ResourcePopup()
    
    @expose(template="openerp.subcontrollers.templates.process")
    def default(self, id=False, res_model=None, res_id=False):

        id = (id or False) and int(id)
        res_id = int(res_id)

        title = _("Select Workflow")
        selection = None

        proxy = rpc.RPCProxy('process.process')
        if id:
            res = proxy.read([id], ['name'], rpc.session.context)[0]
            title = res['name']
        else:
            selection = proxy.search_by_model(res_model, rpc.session.context)
            if res_model and not selection:
                selection = proxy.search_by_model(False, rpc.session.context)

            if len(selection) == 1:
                id, title = selection[0]
                selection = None

        return dict(id=id, res_model=res_model, res_id=res_id, title=title, selection=selection)
    
    @expose('json')
    def get(self, id, res_model=None, res_id=False):

        id = int(id)
        res_id = int(res_id)

        proxy = rpc.RPCProxy('process.process')
        graph = proxy.graph_get(id, res_model, res_id, (80, 80, 150, 100), rpc.session.context)

        related = (res_model or None) and proxy.search_by_model(res_model, rpc.session.context)
        graph['related'] = dict(related or {})

        if graph.get('resource'):
            graph['title'] = _("%(name)s - Resource: %(resource)s, State: %(state)s") % graph
        else:
            graph['title'] = graph['name']

        def update_perm(perm):
            perm = perm or {}

            try:
                perm['date'] = format.format_datetime(perm['write_date'] or perm['create_date'])
            except:
                pass

            perm['text'] = _("Last modified by:")
            perm['value'] = perm.get('write_uid') or perm.get('create_uid')
            if perm['value']: 
                perm['value'] =  '%s (%s)' % (perm['value'][1], perm.get('date') or 'N/A')
            else:
                perm['value'] = 'N/A'

            return perm

        # last modified by
        graph['perm'] = update_perm(graph['perm'] or {})
        for nid, node in graph['nodes'].items():
            if not node.get('res'):
                continue
            node['res']['perm'] = update_perm(node['res']['perm'] or {})

        return graph

    @expose('json')
    def action(self, **kw):
        params, data = TinyDict.split(kw)
        
        button = TinyDict()
        
        button.model = params.model
        button.id = params.id
        button.name = params.action
        button.btype = params.kind
        
        params.button = button

        fobj = form.Form()

        error = ""
        try:
            res = fobj.button_action(params)
        except Exception, e:
            error = str(e)

        return dict(error=error)

    @expose(content_type='application/pdf')
    def print_workflow(self, id, model):
        return actions.execute_report("workflow.instance.graph", ids=[], id=int(id), model=model, nested=False)

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

from turbogears import expose
from turbogears import redirect
from turbogears import controllers

import cherrypy

from openerp import rpc
from openerp import tools
from openerp import common

from openerp.tinyres import TinyResource

class Shortcuts(controllers.Controller, TinyResource):

    def my(self):

        if not rpc.session.is_logged():
            return []
        
        sc = cherrypy.session.get('terp_shortcuts', False)
        if not sc:
            proxy = rpc.RPCProxy('ir.ui.view_sc')
            sc = proxy.get_sc(rpc.session.uid, 'ir.ui.menu', rpc.session.context) or []
            
            cherrypy.session['terp_shortcuts'] = sc
            
        for x in sc:
            if isinstance(x['res_id'], (list, tuple)):
                x['res_id'] = x['res_id'][0]

        return sc
    
    def can_create(self):
        return (rpc.session.is_logged() and 
                rpc.session.active_id and 
                cherrypy.request.path == '/tree/open' and 
                cherrypy.request.params.get('model') == 'ir.ui.menu')

    @expose()
    def default(self):
        from openerp.subcontrollers import actions

        domain = [('user_id', '=', rpc.session.uid), ('resource', '=', 'ir.ui.menu')]
        return actions.execute_window(False, 'ir.ui.view_sc', res_id=None, domain=domain, view_type='form', mode='tree,form')

    @expose()
    def add(self, id):
        id = int(id)
        proxy = rpc.RPCProxy('ir.ui.view_sc')
        
        sc = cherrypy.session.get('terp_shortcuts', False)
        
        if sc:
            for s in sc:
                if s['res_id'] == id:
                    raise redirect('/tree/open', id=id, model='ir.ui.menu')

        name = rpc.RPCProxy('ir.ui.menu').name_get([id], rpc.session.context)[0][1]
        proxy.create({'user_id': rpc.session.uid, 'res_id': id, 'resource': 'ir.ui.menu', 'name': name})

        sc = proxy.get_sc(rpc.session.uid, 'ir.ui.menu', rpc.session.context)
        cherrypy.session['terp_shortcuts'] = sc

        raise redirect('/tree/open', id=id, model='ir.ui.menu')
        
# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

"""
This modules implements custom authorization logic for the OpenERP Web Client.
"""

import time
import types
import re

from turbogears import expose
from turbogears import redirect
from turbogears import config

import cherrypy
import rpc
import pkg_resources

@expose(template="openerp.templates.login")
def _login(target, dblist=None, db= None, user=None, action=None, message=None, origArgs={}):
    """Login page, exposed without any controller, will be used by _check_method wrapper
    """
    url = rpc.session.get_url()
    url = str(url[:-1])

    return dict(target=target, url=url, dblist=dblist, user=user, password=None, 
            db=db, action=action, message=message, origArgs=origArgs)

def secured(fn):
    """A Decorator to make a TinyResource controller method secured.
    """
    def clear_login_fields(kw={}):

        if not kw.get('login_action'):
            return

        if kw.has_key('db'): del kw['db']
        if kw.has_key('user'): del kw['user']
        if kw.has_key('password'): del kw['password']
        if kw.has_key('login_action'): del kw['login_action']

    def get_orig_args(kw={}):
        if not kw.get('login_action'):
            return kw

        new_kw = kw.copy()

        if new_kw.has_key('db'): del new_kw['db']
        if new_kw.has_key('user'): del new_kw['user']
        if new_kw.has_key('password'): del new_kw['password']
        if new_kw.has_key('login_action'): del new_kw['login_action']

        return new_kw

    def wrapper(*args, **kw):
        """The wrapper function to secure exposed methods
        """

        if rpc.session.is_logged():
            # User is logged in; allow access
            clear_login_fields(kw)
            return fn(*args, **kw)
        else:
            # User isn't logged in yet.

            db = None
            user = None
            password = None
            message = None

            action = kw.get('login_action')

            # get some settings from cookies
            try:
                db = cherrypy.request.simple_cookie['terp_db'].value
                user = cherrypy.request.simple_cookie['terp_user'].value
            except:
                pass

            db = kw.get('db', db)
            user = kw.get('user', user)
            password = kw.get('password', password)

            # See if the user just tried to log in
            if rpc.session.login(db, user, password) <= 0:
                # Bad login attempt
                dblist = rpc.session.listdb()
                if dblist == -1:
                    dblist = []
                    message = _("Could not connect to server!")

                if action == 'login':
                    message = _("Bad username or password!")
                
                if config.get('dblist.filter', path='openerp-web'):
                    
                    headers = cherrypy.request.headers
                    host = headers.get('X-Forwarded-Host', headers.get('Host'))

                    base = re.split('\.|:|/', host)[0]                
                    base = base + '_'                
                    dblist = [d for d in dblist if d.startswith(base)]
                    
                return _login(cherrypy.request.path, message=message, dblist=dblist, db=db, 
                        user=user, action=action, origArgs=get_orig_args(kw))

            # Authorized. Set db, user name in cookies
            expiration_time = time.strftime("%a, %d-%b-%Y %H:%M:%S GMT", time.gmtime(time.time() + ( 60 * 60 * 24 * 365 )))
            cherrypy.response.simple_cookie['terp_db'] = db
            cherrypy.response.simple_cookie['terp_user'] = user.encode('utf-8')
            cherrypy.response.simple_cookie['terp_db']['expires'] = expiration_time;
            cherrypy.response.simple_cookie['terp_user']['expires'] = expiration_time;

            # User is now logged in, so show the content
            clear_login_fields(kw)
            return fn(*args, **kw)

    # restore the original values
    wrapper.__name__ = fn.__name__
    wrapper.__doc__ = fn.__doc__
    wrapper.__dict__ = fn.__dict__.copy()
    wrapper.__module__ = fn.__module__

    wrapper.secured = True

    return wrapper

def unsecured(fn):
    """A Decorator to make a TinyResource controller method unsecured.
    """

    def wrapper(*args, **kw):
        return fn(*args, **kw)

    # restore the original values
    wrapper.__name__ = fn.__name__
    wrapper.__doc__ = fn.__doc__
    wrapper.__dict__ = fn.__dict__.copy()
    wrapper.__module__ = fn.__module__

    wrapper.secured = False

    return wrapper

class TinyResource(object):
    """Provides a convenient way to secure entire TG controller
    """
    def __getattribute__( self, name ):
        value= object.__getattribute__(self, name)

        if isinstance(value, types.MethodType ) and hasattr(value, "exposed") and not (hasattr(value, "secured") and not value.secured):
            return secured(value)

        # Some other property
        return value

# vim: ts=4 sts=4 sw=4 si et




# -*- coding: UTF-8 -*-
"""This module contains functions called from console script entry points."""

import os
import sys
import optparse

from os.path import join, dirname, exists

# ubuntu 8.04 has obsoleted `pyxml` package and installs here.
# the path needs to be updated before any `import xml`
_oldxml = '/usr/lib/python%s/site-packages/oldxml' % sys.version[:3]
if exists(_oldxml):
    sys.path.append(_oldxml)

import pkg_resources
try:
    pkg_resources.require("TurboGears >= 1.0.7, < 1.1b1")
except pkg_resources.VersionConflict, e:
    print "Error: TurboGears >= 1.0.7, < 1.1b1 required."
    sys.exit(0)

import turbogears
import cherrypy

cherrypy.lowercase_api = True

from openerp.release import version

class ConfigurationError(Exception):
    pass

def get_config_file():
    setupdir = dirname(dirname(__file__))
    if exists(join(setupdir, "setup.py")):
        return join(setupdir, "dev.cfg")
    return None

def start():
    """Start the CherryPy application server."""

    parser = optparse.OptionParser(version=version)
    parser.add_option("-c", "--config", dest="config", help="specify alternate config file", default=get_config_file())
    (opt, args) = parser.parse_args()

    configfile = opt.config

    if configfile is None:
        try:
            configfile = pkg_resources.resource_filename(
                    pkg_resources.Requirement.parse("openerp-web"), "config/default.cfg")
        except pkg_resources.DistributionNotFound:
            raise ConfigurationError(_("Could not find default configuration."))

    if not exists(configfile):
        raise ConfigurationError(_("Could not find configuration file: %s") % configfile)

    turbogears.update_config(configfile=configfile, modulename="openerp.config")
    
    # save the name of the configfile (TODO: config editor)
    import openerp
    openerp.CONFIG_FILE = configfile
    
    from openerp.controllers import Root

    turbogears.start_server(Root())
    
# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

"""
This module implementes widget parser for form view, and
several widget components.
"""
import xml.dom.minidom

import cherrypy
import turbogears as tg

from openerp import rpc
from openerp import tools
from openerp import cache

from openerp.widgets.interface import TinyInputWidget
from openerp.widgets.interface import TinyCompoundWidget

from openerp.widgets.form import Char
from openerp.widgets.form import Form
from openerp.widgets.form import Frame
from openerp.widgets.form import Button
from openerp.widgets.form import Float
from openerp.widgets.form import Frame
from openerp.widgets.form import DateTime
from openerp.widgets.form import Integer
from openerp.widgets.form import Selection
from openerp.widgets.form import Notebook

class RangeWidget(TinyCompoundWidget):
    template = "openerp.widgets_search.templates.rangewid"

    params = ["field_value"]
    member_widgets = ["from_field", "to_field"]

    def __init__(self, attrs):
        super(RangeWidget, self).__init__(attrs)

        kind = attrs.get('type', 'integer')

        fname = attrs['name']

        from_attrs = attrs.copy()
        to_attrs = attrs.copy()

        from_attrs['name'] = fname + '/from'
        to_attrs['name'] = fname + '/to'

        self.from_field = RANGE_WIDGETS[kind](from_attrs)
        self.to_field = RANGE_WIDGETS[kind](to_attrs)

        # in search view fields should be writable
        self.from_field.readonly = False
        self.to_field.readonly = False

    def set_value(self, value):
        start = value.get('from', '')
        end = value.get('to', '')

        self.from_field.set_value(start)
        self.to_field.set_value(end)

class Search(TinyCompoundWidget):
    template = "openerp.widgets_search.templates.search"
    params = ['fields_type']
    member_widgets = ['_notebook', 'basic', 'advance']

    _notebook = Notebook({}, [])

    def __init__(self, model, domain=[], context={}, values={}):

        super(Search, self).__init__({})

        self.model         = model

        self.domain        = domain
        self.context       = context

        ctx = rpc.session.context.copy()
        view = cache.fields_view_get(self.model, False, 'form', ctx, True)

        dom = xml.dom.minidom.parseString(view['arch'].encode('utf-8'))
        root = dom.childNodes[0]
        attrs = tools.node_attributes(root)
        self.string = attrs.get('string', '')

        self.fields_type = {}
        self.widgets = []
        self.parse(dom, view['fields'], values)

        # also parse the tree view
        view = cache.fields_view_get(self.model, False, 'tree', ctx, True)
        dom = xml.dom.minidom.parseString(view['arch'].encode('utf-8'))
        self.parse(dom, view['fields'], values)

        self.basic = Frame({}, [w for w in self.widgets if not w.adv])
        self.advance = Frame({}, self.widgets)

    def parse(self, root=None, fields=None, values={}):

        for node in root.childNodes:

            if not node.nodeType==node.ELEMENT_NODE:
                continue

            attrs = tools.node_attributes(node)

            if attrs.has_key('colspan'):
                attrs['colspan'] = 1

            if attrs.has_key('nolabel'):
                attrs['nolabel'] = False

            if node.localName in ('form', 'tree'):
                self.parse(root=node, fields=fields, values=values)
                #views += [Frame(attrs, n)]

            elif node.localName == 'notebook':
                self.parse(root=node, fields=fields, values=values)

            elif node.localName == 'page':
                self.parse(root=node, fields=fields, values=values)

            elif node.localName=='group':
                self.parse(root=node, fields=fields, values=values)

            elif node.localName == 'field':
                name = attrs['name']

                if name in self.fields_type:
                    continue

                if not ('select' in attrs or 'select' in fields[name]):
                    continue

                if attrs.get('widget', False):
                    if attrs['widget']=='one2many_list':
                        attrs['widget']='one2many'
                    attrs['type'] = attrs['widget']

                # in search view fields should be writable
                attrs['readonly'] = False
                attrs['required'] = False
                attrs['translate'] = False
                attrs['disabled'] = False
                attrs['visible'] = True
                attrs['invisible'] = False
                attrs['editable'] = True

                try:
                    fields[name].update(attrs)
                except:
                    print "-"*30,"\n malformed tag for:", attrs
                    print "-"*30
                    raise
                
                kind = fields[name]['type']

                if kind not in WIDGETS:
                    continue

                self.fields_type[name] = kind

                field = WIDGETS[kind](attrs=fields[name])
                field.onchange = None
                field.callback = None

                val = fields[name].get('select', False)
                field.adv = val and int(val) > 1

                if kind == 'boolean':
                    field.options = [[1,'Yes'],[0,'No']]
                    field.validator.if_empty = ''

                if values.has_key(name) and isinstance(field, (TinyInputWidget, RangeWidget)):
                    field.set_value(values[name])

                self.widgets += [field]

RANGE_WIDGETS = {
    'date': DateTime,
    'time': DateTime,
    'datetime': DateTime,
    'float': Float,
    'integer': Integer,
}

WIDGETS = {
    'date': RangeWidget,
    'datetime': RangeWidget,
    'float': RangeWidget,
    'integer': RangeWidget,
    'selection': Selection,
    'char': Char,
    'boolean': Selection,
    'text': Char,
    'one2many': Char,
    'one2many_form': Char,
    'one2many_list': Char,
    'many2many': Char,
    'many2one': Char,
    'email' : Char,
    'url' : Char,
}

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import re
import random
import locale
import cherrypy
from base64 import b64encode, b64decode
from StringIO import StringIO

import turbogears as tg

import wikimarkup

from form import Text
from openerp import rpc

_image = re.compile(r'img:(.*)\.(.*)', re.UNICODE)
_rss = re.compile(r'rss:(.*)\.(.*)', re.UNICODE)
_attach = re.compile(r'attach:(.*)\.(.*)', re.UNICODE)
_internalLinks = re.compile(r'\[\[.*\]\]', re.UNICODE)
_edit = re.compile(r'edit:(.*)\|(.*)', re.UNICODE)

class WikiParser(wikimarkup.Parser):
    
    def parse(self, text, id):
        text = text.replace('&nbsp;', 'n-b-s-p')
        text = text.replace('&amp;', 'n-a-m-p')
        text = text.replace('&','&amp;')
        text = text.replace('n-b-s-p', '&nbsp;')
        text = text.replace('n-a-m-p', '&amp;')
        text = text.replace('<code>', '<pre>')
        text = text.replace('</code>', '</pre>')
        
        text = wikimarkup.to_unicode(text)
        text = self.strip(text)
        
        text = super(WikiParser, self).parse(text)
        text = self.addImage(text, id)
        text = self.attachDoc(text, id)
        text = self.recordLink(text)
        text = self.addInternalLinks(text)
        text = self.addRss(text, id)
        return text

    def addRss(self, text, id):
        def addrss(path):
            rssurl = path.group().replace('rss:','')
            import rss.feedparser as feedparser
            data = feedparser.parse(rssurl)
            values = "<h2>%s</h2><br/>" % (data.feed.title)
            values += "%s<br/>" % (data.channel.description)
            for entry in data['entries']:
                values += "<h3><a href='%s'> %s </a></h3><br/>" % (entry.link, entry.title)
                values += "%s <br/>" % (entry.summary)
            
            return values
        
        bits = _rss.sub(addrss, text)
        return bits
    
    def attachDoc(self, text, id):
        def document(path):
            file = path.group().replace('attach:','')
            if file.startswith('http') or file.startswith('ftp') or file.startswith('http'):
                return "<a href='%s'>Download File</a>" % (file)
            else:
                proxy = rpc.RPCProxy('ir.attachment')
                ids = proxy.search([('datas_fname','=',file.strip()), ('res_model','=','wiki.wiki'), ('res_id','=',id)])
                if len(ids) > 0:
                    return "<a href='/wiki/getfile?file=%s&amp;id=%d'>%s</a>" % (file, id, file)
                else:
                    return """<a onclick="openWindow(getURL('/attachment', {model: 'wiki.wiki', id: %d}), 
                    {name : 'Wiki Attachments'})">Attach : %s </a>""" % (id, file)
        bits = _attach.sub(document, text)
        return bits
    
    def addImage(self, text, id):
        def image(path):
            file = path.group().replace('img:','')
            if file.startswith('http') or file.startswith('ftp') or file.startswith('http'):
                return "<img src='%s'/>" % (file)
            else:
                proxy = rpc.RPCProxy('ir.attachment')
                ids = proxy.search([('datas_fname','=',file.strip()), ('res_model','=','wiki.wiki'), ('res_id','=',id)])
                if len(ids) > 0:
                    return "<img src='/wiki/getImage?file=%s&amp;id=%d'/>" % (file, id)
                else:
                    return """<a onclick="openWindow(getURL('/attachment', {model: 'wiki.wiki', id: %d}), 
                    {name : 'Wiki Attachments'})">Attach : %s </a>""" % (id, file)
                #"[[/attachment/?model=wiki.wiki&amp;id=%d | Attach:%s]]" % (id, file)
        bits = _image.sub(image, text) 
        return bits
    
    def recordLink(self, text):
        def record(path):
            record = path.group().replace('edit:','').split("|")
            model = record[0]
            text = record[1].replace('\r','').strip()
            label = "Edit Record"
            if len(record) > 2:
                label = record[2]
            proxy = rpc.RPCProxy(model)
            ids = proxy.name_search(text, [], '=', {})
            if len(ids):
                id = ids[0][0]
            else:
                try:
                    id = int(text)
                except:
                    id = 0
            return "[[/form/edit?model=%s&amp;id=%d | %s]]" % (model, id, label)
        
        bits = _edit.sub(record, text) 
        return bits
    
    def addInternalLinks(self, text):
        from openerp import rpc
        proxy = rpc.RPCProxy('wiki.wiki')
        def link(path):
            link = path.group().replace('[','').replace('[','').replace(']','').replace(']','').split("|")

            mids = proxy.search([('name','ilike',link[0])])
            link_str = ""
            if mids:
               if len(link) == 2:
                   link_str = "<a href='/form/view?model=wiki.wiki&amp;id=%s'>%s</a>" % (mids[0], link[1])
               elif len(link) == 1:
                   link_str = "<a href='/form/view?model=wiki.wiki&amp;id=%s'>%s</a>" % (mids[0], link[0])
            else:
                if len(link) == 2:
                    link_str = "<a href='%s'>%s</a>" % (link[0], link[1])
                elif len(link) == 1:
                    link_str = "<a href='/form/edit?model=wiki.wiki&amp;id=False'>%s</a>" % (link[0])
                    
            return link_str
        
        bits = _internalLinks.sub(link, text) 
        return bits

def wiki2html(text, showToc, id):
    p = WikiParser(show_toc=showToc)
    return p.parse(text, id)

class WikiWidget(Text):
    template = "openerp.widgets.templates.wiki"
    
    params = ["data"]
    css = [tg.widgets.CSSLink('openerp', 'css/wiki.css')]
    javascript = [tg.widgets.JSLink("openerp", "javascript/textarea.js")]

    data = None
    
    def __init__(self, attrs):
        super(WikiWidget, self).__init__(attrs)
        self.data = None

    def set_value(self, value):
        super(WikiWidget, self).set_value(value)
        if value:
            toc = True
            if hasattr(cherrypy.request, 'terp_record'): 
                toc = cherrypy.request.terp_record.get('toc', True)
                params = cherrypy.request.terp_params
                id = params.id
            text = value+'\n\n'
            html = wiki2html(text, toc, id)
            
            self.data = html



###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following 
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be 
#     kept as in original distribution without any changes in all software 
#     screens, especially in start-up page and the software header, even if 
#     the application source code has been changed or updated or code has been 
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
# 
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import turbogears as tg
import cherrypy

from openerp import tools
from openerp import rpc
from openerp import cache

from interface import TinyCompoundWidget

import form
import graph
import listgrid

import tinycalendar

class Screen(TinyCompoundWidget):

    template = """
    <span xmlns:py="http://purl.org/kid/ns#" py:strip="">
        <input type="hidden" id="${name}_terp_model" name="${name}_terp_model" value="${model}"/>
        <input type="hidden" id="${name}_terp_state" name="${name}_terp_state" value="${state}"/>
        <input type="hidden" id="${name}_terp_id" name="${name}_terp_id" value="${str(id)}"/>
        <input type="hidden" id="${name}_terp_ids" name="${name}_terp_ids" value="${str(ids)}"/>
        <input type="hidden" id="${name}_terp_view_ids" name="${name}_terp_view_ids" value="${str(view_ids)}"/>
        <input type="hidden" id="${name}_terp_view_mode" name="${name}_terp_view_mode" value="${str(view_mode)}"/>
        <input type="hidden" id="${name}_terp_view_type" name="${name}_terp_view_type" value="${str(view_type)}"/>
        <input type="hidden" id="${name}_terp_view_id" name="${name}_terp_view_id" value="${str(view_id)}"/>
        <input type="hidden" id="${name}_terp_domain" name="${name}_terp_domain" value="${str(domain)}"/>
        <input type="hidden" id="${name}_terp_context" name="${name}_terp_context" value="${str(context)}"/>
        <input type="hidden" id="${name}_terp_editable" name="${name}_terp_editable" value="${editable}"/>

        <input type="hidden" id="${name}_terp_limit" name="${name}_terp_limit" value="${limit}"/>
        <input type="hidden" id="${name}_terp_offset" name="${name}_terp_offset" value="${offset}"/>
        <input type="hidden" id="${name}_terp_count" name="${name}_terp_count" value="${count}"/>

        <span py:if="widget" py:replace="widget.display(value_for(widget), **params_for(widget))"/>
    </span>
    """

    params = ['model', 'state', 'id', 'ids', 'view_id', 'view_ids', 'view_mode', 'view_type', 'domain', 'context', 'limit', 'offset', 'count']

    member_widgets = ['widget']
    widget = None

    def __init__(self, params=None, prefix='', name='', views_preloaded={}, hastoolbar=False, editable=False, readonly=False, selectable=0, nolinks=1):

        # get params dictionary
        params = params or cherrypy.request.terp_params
        prefix = prefix or (params.prefix or '')

        super(Screen, self).__init__(dict(prefix=prefix, name=name))

        self.model         = params.model
        self.state         = params.state or None
        self.id            = params.id or False
        self.ids           = params.ids
        self.view_ids      = params.view_ids or []
        self.view_mode     = params.view_mode or []
        self.view_type     = params.view_type
        self.view_id       = False
 
        while len(self.view_ids) < len(self.view_mode):
            self.view_ids += [False]

        if not self.view_type and params.view_mode:
            self.view_type = params.view_mode[0]
            
        if self.view_ids and self.view_type in self.view_mode:
            idx = self.view_mode.index(self.view_type)
            self.view_id = self.view_ids[idx]

        self.domain        = params.domain or []
        self.context       = params.context or {}
        self.nodefault     = params.nodefault or False

        self.offset        = params.offset
        self.limit         = params.limit
        self.count         = params.count       

        if (self.ids or self.id) and self.count == 0:
            self.count = rpc.RPCProxy(self.model).search_count(self.domain)

        self.prefix             = prefix
        self.views_preloaded    = views_preloaded or (params.views or {})

        self.hastoolbar         = hastoolbar
        self.toolbar            = None

        self.selectable         = selectable
        self.editable           = editable
        self.readonly           = readonly
        self.link               = nolinks

        # get calendar options
        self.kalendar           = params.kalendar

        if self.view_mode:
            self.add_view_id(self.view_id, self.view_type)

    def add_view_id(self, view_id, view_type):
        self.view_id = view_id

        self.view_id = view_id
        
        if view_type in self.views_preloaded:
            view = self.views_preloaded[view_type]
        else:
            ctx = rpc.session.context.copy()
            ctx.update(self.context)
            view = cache.fields_view_get(self.model, view_id, view_type, ctx, self.hastoolbar)

        self.add_view(view, view_type)

    def add_view(self, view, view_type='form'):

        self.view_id = view.get('view_id', self.view_id)

        if view_type == 'form':
            self.widget = form.Form(prefix=self.prefix,
                                    model=self.model,
                                    view=view,
                                    ids=(self.id or []) and [self.id],
                                    domain=self.domain,
                                    context=self.context,
                                    editable=self.editable,
                                    readonly=self.readonly,
                                    nodefault=self.nodefault, nolinks=self.link)
            
            if not self.ids:
                self.ids = (self.id or []) and [self.id]

        elif view_type == 'tree':
            self.widget = listgrid.List(self.name or '_terp_list',
                                        model=self.model,
                                        view=view,
                                        ids=self.ids,
                                        domain=self.domain,
                                        context=self.context,
                                        editable=self.editable,
                                        selectable=self.selectable,
                                        offset=self.offset, limit=self.limit, count=self.count, nolinks=self.link)

            self.ids = self.widget.ids
            self.count = self.widget.count

        elif view_type == 'graph':
            self.widget = graph.Graph(model=self.model, 
                                      view_id=view.get('view_id', False), 
                                      ids=self.ids, domain=self.domain, 
                                      context=self.context)
            self.ids = self.widget.ids

        elif view_type == 'calendar':
            self.widget = tinycalendar.get_calendar(view=view, 
                                                    model=self.model, 
                                                    ids=self.ids, 
                                                    domain=self.domain, 
                                                    context=self.context, 
                                                    options=self.kalendar)
            
        elif view_type == 'gantt':
            self.widget = tinycalendar.GanttCalendar(model=self.model, 
                                                     view=view, 
                                                     ids=self.ids, 
                                                     domain=self.domain, 
                                                     context=self.context, 
                                                     options=self.kalendar)

        self.string = (self.widget or '') and self.widget.string

        toolbar = {}
        for item, value in view.get('toolbar', {}).items():
            if value: toolbar[item] = value

        self.toolbar = toolbar or None
        self.hastoolbar = (toolbar or False) and True
        
# vim: ts=4 sts=4 sw=4 si et

